import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Constants/fontFamily.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Firebase/sign_in.dart';
import 'package:example/Screens/Admin/AddScreen.dart';
import 'package:example/Screens/Admin/Module.dart';
import 'package:example/Screens/Admin/Permission.dart';
import 'package:example/Screens/Admin/Roles.dart';
import 'package:example/Screens/Admin/Users.dart';
import 'package:example/Screens/ModuleSelectionScreen.dart';
import 'package:example/main.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:universal_platform/universal_platform.dart';

class MainWidgetAdmin extends StatefulWidget {
  MainWidgetAdmin({Key key, this.title, this.id}) : super(key: key);
  final String title;
  int id;

  @override
  MainWidgetAdminState createState() => MainWidgetAdminState();
}

class MainWidgetAdminState extends State<MainWidgetAdmin>
    with TickerProviderStateMixin {
  HiddenDrawerController _drawerController;
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  List<DrawerItem> drawerItem = new List.empty(growable: true);

  int indexValue = 0;

  @override
  void initState() {
    super.initState();

    for (int i = 0; i < ModuleSelectionScreenState.screens.length; i++) {
      if (widget.id == ModuleSelectionScreenState.screens[i].moduleId) {
        drawerItem.add(
          DrawerItem(
            text: Text(
              ModuleSelectionScreenState.screens[i].screenName,
              style: TextStyle(
                  color: Colors.white,
                  fontSize: FontSize.FONT_15,
                  fontFamily: FontFamily.Gilroy,
                  fontStyle: FontStyle.normal,
                  fontWeight: FontWeight.normal),
            ),
            //icon: Icon(Icons.dashboard, color: Colors.white),

            page: getDrawerContent(
              ModuleSelectionScreenState.screens[i].screenName,
              ModuleSelectionScreenState.screens[i].permissionStr,
            ),
          ),
        );
      }
    }
    colorCodePrefs();
    _drawerController = HiddenDrawerController(
       initialPage: getDrawerContent(
          ModuleSelectionScreenState.screens[0].screenName,
          ModuleSelectionScreenState.screens[0].permissionStr),

     /* initialPage: AdminModule( permissionStr: ModuleSelectionScreenState.screens[0].permissionStr),*/
      items: drawerItem,
    );
  }

  Future<void> colorCodePrefs() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
      });
    } else {
      setState(() {
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: HiddenDrawer(
        controller: _drawerController,
        header: Column(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Container(
              alignment: Alignment.topLeft,
              padding: EdgeInsets.only(left: 30),
              child: CircleAvatar(
                radius: 70,
                backgroundColor: Colors.white,
                child: Container(
                  padding: EdgeInsets.all(20),
                  child: Image.asset(
                    ImageAssets.LOGO_WITH_NAME,
                    height: 150,
                  ),
                ),
              ),
            ),
          ],
        ),
        decoration: BoxDecoration(
          gradient: LinearGradient(
            begin: Alignment.topLeft,
            end: Alignment.bottomRight,
            colors: [
              Colors.black87,
              Colors.black87,
              Colors.black87,
            ],
            // tileMode: TileMode.repeated,
          ),
        ),
        logout: () {
          signOutFunction(context);
        },
        versionName: UniversalPlatform.isAndroid
            ? MyApp.version + AppStrings.COPY_RIGHT
            : UrlParams.AV_VALUE + AppStrings.COPY_RIGHT,
      ),
    );
  }

  static Future<void> signOutFunction(BuildContext context) async {
    BUIAndroidAlertBox.showBUIAndroidAlertBox(
        context: context,
        type: AppStrings.ARE_U_SURE,
        enterWidget: Text(AppStrings.DO_U_WANT_LOGOUT),
        isPositiveEnabled: true,
        isNegativeEnabled: true,
        positiveButtonColor: AppColors.primaryColor,
        setPositiveAction: () async {
          await FirebaseAuth.instance.signOut();
          await signOutGoogle(context);
          MyApp.user_token = "";
        },
        positiveButtonTextColor: Colors.white);
  }

  DrawerContent getDrawerContent(String screenName, int permissionStr) {
    int permission = int.parse(permissionStr.toString(), radix: 2);
    switch (screenName) {
      case "Role":
        return AdminRoles(permissionStr: permission);
        break;
      case "User":
        return AdminUser(permissionStr: permission);
        break;
      case "Permission":
        return AdminPermissions(permissionStr: permission);
        break;
      case "Module":
        return AdminModule(permissionStr: permission);
        break;
      case "User":
        return AdminUser(permissionStr: permission);
        break;

      default:
        return AdminUser(permissionStr: permission);
        break;
    }
  }
}
