import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/Admin/module_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Screens/CommonViewDetailScreen.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:transition/transition.dart';

class AdminModule extends DrawerContent {
  AdminModule({Key key, this.permissionStr});

  int permissionStr;

  @override
  _AdminModuleState createState() => _AdminModuleState();
}

class _AdminModuleState extends State<AdminModule> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  bool visible = true;
  String dateTime;
  String currency;
  String ownerId;

  bool isLoader = true;

  ModuleDetailsBean moduleDetailsBean;
  List<ModuleDetailListBean> moduleDetailsListBean =
      new List.empty(growable: true);

  ScrollController _scrollController = new ScrollController();
  int startIndex = 0;
  int offset = 10;

  bool isLoading = false;

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    colorCodePref();

    //widget.permissionStr = int.parse("10000", radix: 2);

    permissionMethod();

    callModuleDetailsAPI();

    _scrollController.addListener(() {
      if (_scrollController.position.pixels ==
          _scrollController.position.maxScrollExtent) {
        if (moduleDetailsListBean.length >= 10) {
          startIndex = startIndex + offset;
          callModuleDetailsAPI();
        } else {
          startIndex = startIndex;
        }
      }
    });
    super.initState();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    String dateTme = prefs.getString(AppStrings.date);
    String currencySymbol = prefs.getString(AppStrings.currency);

    dateTime = dateTme;
    currency = currencySymbol;
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: _scaffoldKey,
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: false,
              ),
              isLoader
                  ? Stack(
                      children: [
                        isLoaderView,
                      ],
                    )
                  : Stack(
                      children: [deleteDetailsView, detailsView],
                    )
            ],
          ),
        ),
        floatingActionButton: Visibility(
          visible: visible,
          child: FloatingActionButton(
            mini: true,
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child: CommonViewDetailScreen(
                        primaryColor: primaryColor,
                        secondaryColor: secondaryColor,
                        pageId: AppStrings.permission,
                        ownerId: ownerId ?? "7",
                        title: AppStrings.create + AppStrings.permission,
                        commonDD: [1, 2, 3],
                        onCallBack: () {},
                        permissionStr: widget.permissionStr,
                      ),
                      transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
            },
            child: const Icon(Icons.add),
            backgroundColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        AppStrings.holdToDelete,
        style: TextStyle(fontSize: 14, color: Colors.black87),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get isLoaderView {
    /*return Container(
      height: MediaQuery.of(context).size.height * 0.80,
      margin: EdgeInsets.only(top: 30),


      alignment: Alignment.center,
      padding: EdgeInsets.all(8),
      child: GridView.builder(
          gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
              maxCrossAxisExtent: 200,
              childAspectRatio: 3 / 2,
              crossAxisSpacing: 1,
              mainAxisSpacing: 1),
          itemCount: 20,
          itemBuilder: (BuildContext ctx, index) {
            return Card(
              elevation: 10,
              shape: BeveledRectangleBorder(
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                     )),
              child: Container(
                height: 50,
                padding: EdgeInsets.all(3),
                child: CardPlaceHolderWithAvatar(
                  height: 10,
                ),
              ),
            );
          }),
    );*/
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Widget get detailsView {
    if (moduleDetailsListBean.length != 0) {
      return RefreshIndicator(
        color: primaryColor,
        onRefresh: () async {
          startIndex = 0;
          moduleDetailsListBean =
              List<ModuleDetailListBean>.empty(growable: true);
          return await callModuleDetailsAPI();
        },
        child: Container(
          height: MediaQuery.of(context).size.height * 0.8,
          margin: EdgeInsets.only(top: 35),
          child: ListView.builder(
              itemCount: moduleDetailsListBean.length,
              itemBuilder: (BuildContext ctx, index) {
                return InkWell(
                    onLongPress: () {
                      if (moduleDetailsListBean[index].status == "active") {
                        BUIAndroidAlertBox.showBUIAndroidAlertBox(
                          context: context,
                          type: "Warning",
                          enterWidget: Text(AppStrings.areYouSureToDelete +
                              moduleDetailsListBean[index].moduleName),
                          isPositiveEnabled: true,
                          isNegativeEnabled: true,
                          setPositiveAction: () {
                            setState(() {
                              moduleDetailsListBean.removeAt(index);
                            });

                            /*  callModuleDeleteAPI(moduleDetailsListBean[index].id
                              .toString());*/
                          },
                        );
                      } else {
                        Utils.showInSnackBar(_scaffoldKey, AppStrings.noAccess);
                      }
                    },
                    radius: 30,
                    enableFeedback: true,
                    splashColor: AppColors.halfWhite,
                    onTap: () {
                      if (moduleDetailsListBean[index].status == "active") {
                        Utils.showInSnackBar(_scaffoldKey,
                            moduleDetailsListBean[index].moduleName);
                      } else {
                        Utils.showInSnackBar(_scaffoldKey, AppStrings.noAccess);
                      }
                    },
                    child: BUICard.imageWithTextCard(
                      isVisible: true,
                      cardText: moduleDetailsListBean[index].moduleName,
                      //cardText: moduleName[index].moduleName.length > 0 ?'${moduleName[index].moduleName[0].toUpperCase()}${moduleName[index].moduleName.substring(1)}':'',
                      cardTextColor: AppColors.primaryColor,
                      cardType: CardType.cardWithImageAndText,
                      sourceImage: AppStrings.vegImage,
                      //cardColor: primaryColor,
                      isEnabled: moduleDetailsListBean[index].status == "active"
                          ? true
                          : false,

                      isGrid: false,
                      height: 100,
                    ));
              }),
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Future<void> callModuleDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.adminModuleViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    ModuleDetailsBean detailsBean = ModuleDetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {}
    if (detailsBean.status) {
      moduleDetailsBean = detailsBean;
      moduleDetailsListBean = detailsBean.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callModuleDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.adminModuleDelete;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        },
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      callModuleDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }
}
