import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_loading_widget.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/module_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Screens/CommonViewDetailScreen.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import '../../main.dart';

class AdminPermissions extends DrawerContent {
  AdminPermissions({Key key, this.permissionStr});

  int permissionStr;

  @override
  _AdminPermissionsState createState() => _AdminPermissionsState();
}

class _AdminPermissionsState extends State<AdminPermissions> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  bool visible = true;
  String dateTime;
  String currency;
  String ownerId;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool isLoader = true;

  List<Modules> moduleName = new List.empty(growable: true);

  ModuleBean moduleBean;

  static List<Screens> screens = new List.empty(growable: true);

  String userID;

  String moduleNameStr;

  @override
  void initState() {
    colorCodePref();

    //widget.permissionStr = int.parse("10000", radix: 2);

    permissionMethod();

    super.initState();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);

    userID = prefs.getString(AppStrings.USER_ID);
    callUserAPI();
    setState(() {
      ownerId = ownerIdValue;
    });

    String dateTme = prefs.getString(AppStrings.date);
    String currencySymbol = prefs.getString(AppStrings.currency);

    dateTime = dateTme;
    currency = currencySymbol;
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: _scaffoldKey,
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: false,
              ),
              isLoader
                  ? Stack(
                      children: [
                        deleteDetailsView,
                        isLoaderView,
                      ],
                    )
                  : Stack(
                      children: [deleteDetailsView, detailsView],
                    )
            ],
          ),
        ),
        floatingActionButton: Visibility(
          visible: false,
          child: FloatingActionButton(
            mini: true,
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child: CommonViewDetailScreen(
                        primaryColor: primaryColor,
                        secondaryColor: secondaryColor,
                        pageId: AppStrings.users,
                        ownerId: ownerId ?? "7",
                        commonDD: [1, 2, 3],
                        title: AppStrings.create + AppStrings.users,
                        permissionStr: widget.permissionStr,
                        onCallBack: () {},
                      ),
                      transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
            },
            child: const Icon(Icons.add),
            backgroundColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        AppStrings.delete_and_edit,
        style: TextStyle(fontSize: 14, color: Colors.black87),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get detailsView {
    if (screens.length != 0) {
      return RefreshIndicator(
        color: primaryColor,
        onRefresh: () async {
          return await callUserAPI();
        },
        child: Container(
          height: MediaQuery.of(context).size.height * 0.80,
          margin: EdgeInsets.only(top: 30),
          child: ListView.builder(
              itemCount: screens.length,
              // Add one more item for progress indicator
              padding: EdgeInsets.symmetric(vertical: 8.0),
              itemBuilder: (BuildContext context, int index) {
                return buildUsageItems(context, index);
              }),
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Widget buildUsageItems(BuildContext context, int index) {
    return Dismissible(
      key: Key(""),
      background: slideRightBackground(),
      secondaryBackground: slideLeftBackground(),
      confirmDismiss: (direction) async {
        if (direction == DismissDirection.endToStart) {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this ${screens[index].screenName}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  screens.removeAt(index);
                });
              },
            );
            return res;
          } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
            Toast.show("viewAccessForOwn", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForOwn) {
            Toast.show("viewAndAddAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            Toast.show("fullAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
            Toast.show("viewAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForAllUser) {
            Toast.show("viewAndAddAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr == AppStrings.noAccessPermission) {
            Toast.show("noAccessPermission", context);
            return false;
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        } else {
          return false;
        }
      },
      child: InkWell(
        onTap: () {},
        child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: screens[index].screenName ?? "",
          cardName: //moduleNameStr,

              screens[index].moduleId.toString() ?? "",
          statusTag: screens[index].permissionStr.toString() ?? "",
          dashBoardCardTime: dateTime == "dd-mm-yyyy" && dateTime != null
              ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(screens[index].createdAt)}" ??
                  ""
              : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(screens[index].createdAt)}" ??
                  "",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  Future<void> callUserAPI() async {
    setState(() {
      isLoader = true;
    });

    String urlController = ServerUrl.buiUserPermission;
    var objToSend = {
      "\"condition\"": [
        {
          "\"${"uid"}\"": "\"${userID.toString()}\"",
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    ModuleBean moduleBeanResponse = ModuleBean.fromJson(map);

    if (moduleBeanResponse.statusCode == 400) {}
    if (moduleBeanResponse.status) {
      moduleBean = moduleBeanResponse;
      moduleName = moduleBeanResponse.data.modules;
      screens = moduleBeanResponse.data.screens;

      for (int i = 0; i < moduleBeanResponse.data.modules.length; i++) {
        for (int j = 0; j < moduleBeanResponse.data.screens.length; j++) {
          if (moduleBeanResponse.data.screens[j].moduleId ==
              moduleBeanResponse.data.modules[i].id) {
            moduleNameStr = moduleBeanResponse.data.modules[i].moduleName;
          }
        }
      }
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }
}
