import 'dart:convert';

import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Screens/CRM/AccountViewDetailScreen.dart';
import 'package:example/Screens/CRM/FilterScreen.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/platform.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import 'DrawerMenuHeaderDesign.dart';

class AccountScreen extends DrawerContent {
  AccountScreen({Key key, this.title, this.permissionStr});

  final String title;
  int permissionStr;

  @override
  _AccountScreenState createState() => _AccountScreenState();
}

class _AccountScreenState extends State<AccountScreen> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  AccountDetailsBean accountDetailsBean;

  static List<AccountListBean> accountListBeanData =
      List<AccountListBean>.empty(growable: true);

  bool isLoader = true;

  String jsonFormFilterData;
  List<dynamic> industry = List<dynamic>.empty(growable: true);

  List<String> actions = ["Edit", "Delete"];

  List<dynamic> status = List<dynamic>.empty(growable: true);

  String dateTime;
  String currency;
  var now = DateTime.now();

  bool isLoading = false;

  ScrollController _scrollController = new ScrollController();
  int startIndex = 0;
  int offset = 10;

  bool visible = true;

  @override
  void initState() {
    colorCodePref();

    //widget.permissionStr = int.parse("10000", radix: 2);

    permissionMethod();
    accountScreenViewDetailsAPI();
    _scrollController.addListener(() {
      if (_scrollController.position.pixels ==
          _scrollController.position.maxScrollExtent) {
        if (accountListBeanData.length >= 10) {
          startIndex = startIndex + offset;
          accountScreenViewDetailsAPI();
        } else {
          startIndex = startIndex;
        }
      }
    });
    super.initState();
  }

  @override
  void dispose() {
    _scrollController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: true,
                isFilterClicked: onFilterClicked,
              ),
              isLoader
                  ? Stack(
                      children: [
                        deleteDetailsView,
                        isLoaderView,
                      ],
                    )
                  : Stack(
                      children: [deleteDetailsView, detailsView],
                    )
            ],
          ),
        ),
        floatingActionButton: Visibility(
          visible: visible,
          child: FloatingActionButton(
            mini: true,
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child: AccountViewDetailScreen(
                        pageTitle: "Info",
                        type: "Add",
                        onCallBack: onAPICallBack,
                        theme: primaryColor,
                        dateTime: dateTime,
                        currency: currency,
                        secondaryColor: secondaryColor,
                      ),
                      transitionEffect: TransitionEffect.SCALE));
            },
            child: const Icon(Icons.add),
            backgroundColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
          AppStrings.delete_and_edit,


          style: MyApp.textTheme.bodyText1.copyWith(  color: Colors.black87,
              fontSize: FontSize.FONT_12,
              //fontStyle: FontStyle.italic,
              fontWeight: FontWeight.bold,
              letterSpacing: 1,
              wordSpacing: 1
          )
      ),
      color: Colors.grey[400],
    );
  }

  Widget get detailsView {
    if (accountListBeanData.length != 0) {
      return RefreshIndicator(
        color: primaryColor,
        onRefresh: () async {
          startIndex = 0;
          accountListBeanData = List<AccountListBean>.empty(growable: true);
          return await accountScreenViewDetailsAPI();
        },
        child: Container(
          height: MediaQuery.of(context).size.height * 0.80,
          margin: EdgeInsets.only(top: 30),
          child: ListView.builder(
            itemCount: accountListBeanData.length +
                1, // Add one more item for progress indicator
            padding: EdgeInsets.symmetric(vertical: 8.0),
            itemBuilder: (BuildContext context, int index) {
              if (index == accountListBeanData.length) {
                return _buildProgressIndicator();
              } else {
                return buildUsageItems(context, index);
              }
            },
            controller: _scrollController,
          ),
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Future<void> callAccountDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmDeleteAccount;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        },
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      accountScreenViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
    Utils.hideKeyboard(context);
    //item.remove();
    //accountScreenViewDetailsAPI();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    String dateTme = prefs.getString(AppStrings.date);
    String currencySymbol = prefs.getString(AppStrings.currency);

    dateTime = dateTme;
    currency = currencySymbol;
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  Future<void> onAPICallBack() async {
    isLoader = false;

    setState(() {
      accountScreenViewDetailsAPI();
      this.mounted;
    });
  }

  Future<void> onFilterClicked() async {
    colorCodePref();
    setState(() {
      Navigator.push(
          context,
          Transition(
              child: FilterScreen(
                primaryColor: primaryColor,
                secondaryColor: secondaryColor,
                filterFormData: jsonFormFilterData,
                fromPageId: "accountId",
                isActionApply: (String obj) {
                  print("Object=====" + obj.toString());
                  callFilterAPI(obj);
                },
              ),
              transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
    });
  }

  Future<void> accountScreenViewDetailsAPI() async {
    setState(() {
      if (startIndex == 0)
        isLoader = true;
      else
        isLoading = true;
    });
    String urlController = ServerUrl.crmAccountViewDetails;

    var objToSend = {
      "\"condition\"": [],
      /*"\"page\"": {
        "\"orderby\"": [
          {
            "\"id\"": "\"${"asc"}\"",
          }
        ],
        "\"no\"": "\"${startIndex.toString()}\"",
        "\"size\"": "\"${"10"}\"",
      },*/
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    AccountDetailsBean accountDetailsResponseBean =
        AccountDetailsBean.fromJson(map);

    if (accountDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }

    if (startIndex == 0 && accountDetailsResponseBean.data == null) {
      setState(() {
        isLoader = true;
      });
    }
    if (accountDetailsResponseBean.status) {
      accountDetailsBean = accountDetailsResponseBean;
      accountListBeanData = accountDetailsResponseBean.data;
      if (accountDetailsResponseBean.data != null)
        //  accountListBeanData.addAll(accountDetailsResponseBean.data);

        status = [
          for (int i = 0; i < accountListBeanData.length; i++)
            {
              'label': accountListBeanData[i].statusid.name.toString(),
              'value': accountListBeanData[i].statusid.id.toString(),
            },
        ];

      final jsonList = status.map((item) => jsonEncode(item)).toList();

      final uniqueJsonList = jsonList.toSet().toList();

      status = uniqueJsonList.map((item) => jsonDecode(item)).toList();

      industry = [
        for (int i = 0; i < accountListBeanData.length; i++)
          if (accountListBeanData[i].industry != null)
            {
              'label': accountListBeanData[i].industry.name.toString(),
              'value': accountListBeanData[i].industry.id.toString(),
            },
      ];

      final jsonList1 = industry.map((item) => jsonEncode(item)).toList();

      final uniqueJsonList1 = jsonList1.toSet().toList();

      industry = uniqueJsonList1.map((item) => jsonDecode(item)).toList();

      status.insert(0, {
        'label': '',
        'value': '',
      });
      industry.insert(0, {
        'label': '',
        'value': '',
      });

      jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'statusid',
            'key': 'select1',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'readOnly': true,
            'value': '6',
            'items': status ??
                [
                  {'label': "Select", 'value': ""},
                  {"label": "New", "value": "6"},
                  {"label": "Active", "value": "7"},
                  {"label": "Inactive", "value": "8"},
                ]
          },
          {
            'apiKeyValue': 'industry',
            'key': 'select1',
            'type': 'select',
            'label': 'Industry',
            'required': false,
            'value': '',
            'items': industry ??
                [
                  {'label': "Software", 'value': "41"},
                  {'label': "Hardware", 'value': "42"},
                  {'label': "Consumar goods", 'value': "43"},
                  {'label': "Farm & Farm Equipments", 'value': "44"},
                  {'label': "Manufacturing", 'value': "45"},
                  {'label': "Foundry", 'value': "46"},
                  {'label': "Automobile", 'value': "47"},
                  {'label': "Textile", 'value': "48"},
                  {'label': "Others", 'value': "49"},
                ],
          },
        ]
      });

      setState(() {
        if (startIndex == 0)
          isLoader = false;
        else
          isLoading = false;
      });
    }
  }

  Future<void> callFilterAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmAccountViewDetails;
    var objToSend = {
      "\"condition\"": [obj],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    AccountDetailsBean accountDetailsResponseBean =
        AccountDetailsBean.fromJson(map);

    if (accountDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }

    if (accountDetailsResponseBean.status) {
      accountDetailsBean = accountDetailsResponseBean;
      accountListBeanData = accountDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  Widget _buildProgressIndicator() {
    return new Padding(
      padding: const EdgeInsets.all(8.0),
      child: new Center(
        child: new Opacity(
          opacity: isLoading ? 1.0 : 00,
          //child: new CircularProgressIndicator(),
          child: Loader(
            loaderColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget buildUsageItems(BuildContext context, int index) {
    return Dismissible(
      key: Key(accountListBeanData[index].id.toString()),
      background: slideRightBackground(),
      secondaryBackground: slideLeftBackground(),
      confirmDismiss: (direction) async {
        if (direction == DismissDirection.endToStart) {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this ${accountListBeanData[index].companyname}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  accountListBeanData.removeAt(index);
                });

                callAccountDeleteAPI(accountListBeanData[index].id.toString());
              },
            );
            return res;
          } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
            Toast.show("viewAccessForOwn", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForOwn) {
            Toast.show("viewAndAddAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            Toast.show("fullAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
            Toast.show("viewAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForAllUser) {
            Toast.show("viewAndAddAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr == AppStrings.noAccessPermission) {
            Toast.show("noAccessPermission", context);
            return false;
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        } else {
          return Navigator.of(context).push(MaterialPageRoute(
            builder: (context) {
              return AccountViewDetailScreen(
                pageTitle: "Info",
                dateTime: dateTime,
                currency: currency,
                id: accountDetailsBean.data[index].id,
                accountListBeanData: accountDetailsBean.data[index],
                onCallBack: onAPICallBack,
                theme: primaryColor,
                secondaryColor: secondaryColor,
                permissionStr: widget.permissionStr,
                visible: visible,
              );
            },
          ));
        }
      },
      child: InkWell(
        onTap: () {
          Navigator.push(
              context,
              Transition(
                  child: AccountViewDetailScreen(
                    pageTitle: "Info",
                    dateTime: dateTime,
                    currency: currency,
                    id: accountDetailsBean.data[index].id,
                    accountListBeanData: accountDetailsBean.data[index],
                    onCallBack: onAPICallBack,
                    theme: primaryColor,
                    secondaryColor: secondaryColor,
                    permissionStr: widget.permissionStr,
                    visible: visible,
                  ),
                  transitionEffect: TransitionEffect.SCALE));
        },
        child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: accountListBeanData[index].email ?? "",
          cardName: accountListBeanData[index].companyname ?? "",
          statusTag: accountListBeanData[index].statusid.name ?? "",
          dashBoardCardTime: dateTime == "dd-mm-yyyy" && dateTime != null
              ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(accountListBeanData[index].createdAt)}" ??
                  ""
              : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(accountListBeanData[index].createdAt)}" ??
                  "",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }
}
