import 'dart:convert';

import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/notes_details_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Screens/CRM/CreateNoteScreen.dart';
import 'package:example/Screens/CRM/CreateTaskScreen.dart';
import 'package:example/Screens/platform.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:transition/transition.dart';

// ignore: must_be_immutable
class AccountViewDetailScreen extends StatefulWidget {
  String pageTitle;
  String dateTime;
  String currency;
  String email;
  String firstName;
  String type;
  Color theme;
  Color secondaryColor;
  int id;
  Function() onCallBack;

  AccountListBean accountListBeanData;
  int permissionStr;
  bool visible;

  AccountViewDetailScreen({
    Key key,
    this.id,
    this.type,
    this.pageTitle,
    this.accountListBeanData,
    this.theme,
    this.onCallBack,
    this.secondaryColor,
    this.dateTime,
    this.currency,
    this.permissionStr,
    this.visible,
  }) : super(key: key);

  @override
  AccountViewDetailScreenState createState() => AccountViewDetailScreenState(
      pageTitle: pageTitle,
      email: email,
      firstName: firstName,
      id: id,
      accountListBeanData: accountListBeanData);
}

class AccountViewDetailScreenState extends State<AccountViewDetailScreen> {
  String pageTitle;
  String email;
  String firstName;
  int id;
  AccountListBean accountListBeanData;
  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List.empty(growable: true);
  NotesDetailsBean notesDetailsBean;
  List<NotesListDetailsBean> notesListDetailsBean =
      new List.empty(growable: true);

  dynamic formGeneral;

  List<dynamic> status = new List.empty(growable: true);
  List<dynamic> industry = new List.empty(growable: true);

  AccountViewDetailScreenState(
      {this.pageTitle,
      this.email,
      this.firstName,
      this.id,
      this.accountListBeanData});

  String form;

  dynamic response;

  Color theme = AppColors.APP_COLOR_PRIMARY;

  bool isLoader = true;
  bool isLoading = true;

  SharedPreferences prefs;

  Map keyBoardTypes = {
    'number': TextInputType.number,
    'text': TextInputType.text,
    'email': TextInputType.emailAddress,
    'url': TextInputType.emailAddress,
  };

  bool showFloating = false;
  int indexValue;

  List<String> taskName = ["Open", "Close"];

  bool flag = true;

  @override
  void initState() {
    super.initState();

    accountMastViewAPI();
    initSharedPref();
    if (widget.type != "Add") {
      taskAPI();
      notesAPI();
    }
  }

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  String ownerId;

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldKey,
      appBar: widget.type == "Add"
          ? AppBar(
              title: Text("Add Account"),
              backgroundColor: widget.theme,
            )
          : null,
      body: widget.type != "Add"
          ? DefaultTabController(
              length: 3,
              child: NestedScrollView(
                  headerSliverBuilder:
                      (BuildContext context, bool innerBoxIsScrolled) {
                    return <Widget>[
                      SliverAppBar(
                        expandedHeight: 150.0,
                        floating: true,
                        pinned: true,
                        title: widget.type != "Add"
                            ? Text(widget.accountListBeanData.companyname)
                            : Text(widget.pageTitle),
                        /*Text(widget.accountListBeanData.companyname != null
                      ? widget.accountListBeanData.companyname
                      : widget.pageTitle),*/
                        backgroundColor:
                            widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                        bottom: TabBar(
                          onTap: (index) {
                            indexValue = index;
                            if (index == 1) {
                              setState(() {
                                showFloating = true;
                              });
                            } else if (index == 2) {
                              setState(() {
                                showFloating = true;
                              });
                            } else {
                              setState(() {
                                showFloating = false;
                              });
                            }
                          },
                          indicatorColor:
                              widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                          labelColor: Colors.black87,
                          unselectedLabelColor: Colors.grey,
                          tabs: [
                            Tab(
                              child: Container(
                                child: Text(
                                  AppStrings.ACCOUNT_INFO,
                                  style: TextStyle(
                                      color: AppColors.APP_COLOR_WHITE,
                                      fontSize: FontSize.FONT_15,
                                      fontWeight: FontWeight.bold),
                                ),
                              ),
                            ),
                            Tab(
                              child: Container(
                                child: Text(
                                  AppStrings.TASK,
                                  style: TextStyle(
                                      color: AppColors.APP_COLOR_WHITE,
                                      fontSize: FontSize.FONT_15,
                                      fontWeight: FontWeight.bold),
                                ),
                              ),
                            ),
                            Tab(
                              child: Container(
                                child: Text(
                                  AppStrings.NOTES,
                                  style: TextStyle(
                                      color: AppColors.APP_COLOR_WHITE,
                                      fontSize: FontSize.FONT_15,
                                      fontWeight: FontWeight.bold),
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),
                    ];
                  },
                  body: isLoading
                      ? Center(
                          child: Loader(
                          loaderColor: widget.theme,
                        ))
                      : TabBarView(
                          physics: NeverScrollableScrollPhysics(),
                          children: [
                            _formView,
                            RefreshIndicator(
                                color: widget.theme,
                                onRefresh: () {
                                  return taskAPI();
                                },
                                child: _taskView),
                            RefreshIndicator(
                                color: widget.theme,
                                onRefresh: () {
                                  return notesAPI();
                                },
                                child: _notesView),
                          ],
                        )),
            )
          : isLoading
              ? Center(
                  child: Loader(
                  loaderColor: theme,
                ))
              : Container(child: _formView),
      floatingActionButton: showFloating
          ? Visibility(
              visible: widget.visible,
              child: FloatingActionButton(
                mini: true,
                onPressed: () {
                  if (indexValue == 1) {
                    Navigator.push(
                        context,
                        Transition(
                            child: CreateTaskScreen(
                              id: widget.id.toString(),
                              primaryColor: widget.theme,
                              secondaryColor: widget.secondaryColor,
                              onCallBack: onTaskAPICall,
                              fromPageId: 'accountid',
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  } else {
                    Navigator.push(
                        context,
                        Transition(
                            child: CreateNoteScreen(
                              id: widget.id.toString(),
                              primaryColor: widget.theme,
                              secondaryColor: widget.secondaryColor,
                              onCallBack: onNotesAPICall,
                              fromPageId: 'accountid',
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                  }
                },
                child: const Icon(Icons.add),
                backgroundColor: widget.theme ?? Colors.green,
              ))
          : null,
    );
  }

  Widget get _formView {
    return new ListView(
      children: [
        isLoading
            ? Center(
                child: Loader(
                loaderColor: theme,
              ))
            : Form(
                child: new Center(
                  child: new Column(children: <Widget>[
                    new JsonSchema(
                      primaryColor: widget.theme ?? AppColors.primaryColor,
                      secondaryColor:
                          widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                      keyboardTypes: keyBoardTypes,
                      form: form,
                      onChanged: (dynamic response) {
                        this.response = response;
                      },
                      buttonText: widget.type == AppStrings.ADD
                          ? AppStrings.ADD
                          : AppStrings.SAVE,
                      actionSave: (data) {
                        Utils.hideKeyboard(context);
                        setState(() {
                          formGeneral = data;
                        });

                        if (widget.permissionStr ==
                                AppStrings.fullAccessForAll ||
                            widget.permissionStr ==
                                AppStrings.fullAccessForOwn) {
                          var jsonObj;
                          if (widget.type != "Add") {
                            jsonObj = {
                              for (int i = 0; i < data['fields'].length; i++)
                                json.encode(data['fields'][i]['apiKeyValue']
                                        .toString()):
                                    json.encode(
                                        data['fields'][i]['value'].toString())
                            };
                          } else {
                            jsonObj = {
                              for (int i = 0; i < data['fields'].length; i++)
                                json.encode(data['fields'][i]['apiKeyValue']
                                        .toString()):
                                    json.encode(
                                        data['fields'][i]['value'].toString()),
                              "\"${UrlParams.AID}\"":
                                  "\"${UrlParams.AID_VALUE}\"",
                              "\"${UrlParams.AKEY}\"":
                                  "\"${UrlParams.AKEY_VALUE}\"",
                              "\"${UrlParams.AV}\"":
                                  "\"${UrlParams.AV_VALUE}\"",
                            };
                          }
                          if (widget.type != "Add") {
                            updateAccountAPI(jsonObj.toString());
                          } else {
                            addAccountNewAPI(jsonObj.toString());
                          }
                        } else {
                          Utils.showInSnackBar(
                              _scaffoldKey, AppStrings.noAccess);
                        }
                      },
                      actionCancel: () {
                        Navigator.pop(context);
                      },
                      buttonSave: true,
                    ),
                  ]),
                ),
              ),
      ],
    );
  }

  Widget get _taskView {
    if (taskListBeanData.length != 0) {
      return ListView.builder(
        itemCount: taskName.length,
        itemBuilder: (_, index) {
          return ExpansionTile(
              title: Text(
                taskName[index].toString() == "Open"
                    ? "UpComing Task"
                    : taskName[index].toString() == "Close"
                        ? "Completed Task"
                        : taskName[index].toString(),
                style: TextStyle(
                    color: AppColors.APP_COLOR_BLACK,
                    fontSize: FontSize.FONT_17,
                    fontWeight: FontWeight.bold),
              ),
              children: [
                ...taskListBeanData.map((e) {
                  if (e.statusid.name == taskName[index]) {
                    return BUICard.taskCard(
                      cardStatus: e.activitytypeid.name,
                      cardDesc: e.desc,
                      cardDate: widget.dateTime == "dd-mm-yyyy" &&
                              widget.dateTime != null
                          ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(e.duedate)}" ??
                              ""
                          : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(e.duedate)}" ??
                              "",
                      cardType: CardType.cardTask,
                      cardSize: CardSize.medium,
                      cardColor: Colors.white,
                      cardTextColor: Colors.black87,
                      width: MediaQuery.of(context).size.width,
                      padding: 5,
                      height: 140,
                      isActiveStatus: e.statusid.name == "Open" ? false : true,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(5.0),
                      ),
                      elevation: 10,
                      secondaryColor: widget.secondaryColor,
                      onStateChange: () {
                        setState(() {
                          if (e.statusid.name == "Open") {
                            e.statusid.name = "Close";
                            Utils.showInSnackBar(
                                _scaffoldKey, "Move into Close state");
                          } else {
                            Utils.showInSnackBar(_scaffoldKey,
                                "You cannot change the task from the closed position to the open position. Please contact admin");
                          }
                        });
                      },
                    );
                  } else {
                    return Container();
                  }
                }).toList(),
              ]);
        },
      );
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_TASK_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Widget get _notesView {
    if (notesListDetailsBean.length != 0) {
      return ListView.builder(
          padding: const EdgeInsets.all(8),
          itemCount: notesListDetailsBean.length,
          itemBuilder: (BuildContext context, int index) {
            return BUICard.notesCard(
              cardType: CardType.cardNotes,
              cardSize: CardSize.medium,
              cardColor: Colors.white,
              cardTextColor: Colors.black87,
              width: MediaQuery.of(context).size.width,
              padding: 2,
              height: 123,
              cardAuthor: widget.accountListBeanData.companyname.toUpperCase(),
              cardDesc: notesListDetailsBean[index].desc,
              cardDate: widget.dateTime == "dd-mm-yyyy" &&
                      widget.dateTime != null
                  ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(notesListDetailsBean[index].updatedAt)}" ??
                      ""
                  : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(notesListDetailsBean[index].updatedAt)}" ??
                      "",
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(5.0),
              ),
              elevation: 10,
              secondaryColor: widget.secondaryColor,
              flag: flag,
              onStateChange: () {
                setState(() {
                  flag = !flag;
                });
              },
            );

            /*return BUICard(
              cardType: CardType.customCard,
              cardText: notesListDetailsBean[index].desc.toString(),
              cardSize: CardSize.small,
              height: MediaQuery.of(context).size.height * 0.2,
              cardColor: Colors.white,
              elevation: 20,
              padding: 15,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(15.0),
              ),
            );*/
          });
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_NOTES_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Future<void> updateAccountAPI(String jsonString) async {
    String urlController = ServerUrl.crmUpdateAccount;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": id.toString()}
      ],
      "\"data\"": [jsonString.toString()],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> accountMastViewAPI() async {
    setState(() {
      isLoader = true;
      isLoading = true;
    });
    String urlController = ServerUrl.crmMasterDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {}
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Account")
            if (detailsBean.data[i].key == "Status")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];
      industry = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Industry")
            {
              'label': detailsBean.data[i].name,
              'value': detailsBean.data[i].id.toString(),
            },
      ];
      status.insert(0, {
        'label': '',
        'value': '',
      });
      industry.insert(0, {
        'label': '',
        'value': '',
      });

      if (widget.type != "Add") {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'companyname',
              'key': 'text',
              'type': 'input',
              'label': 'Company Name',
              'required': true,
              'readOnly': true,
              'value': accountListBeanData.companyname.toString() != "null"
                  ? accountListBeanData.companyname.toString()
                  : "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'readOnly': false,
              'value': accountListBeanData.statusid.id.toString(),
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'industry',
              'key': 'Industry',
              'type': 'select',
              'label': 'Industry',
              'required': true,
              'value': accountListBeanData.industry.id.toString() ?? "",
              'items': industry != null
                  ? industry
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'noofemployees',
              'key': 'number',
              'type': 'input',
              'label': 'No of Employees',
              'required': true,
              'hint': 'Select a status',
              'value': accountListBeanData.noofemployees.toString() != "null"
                  ? accountListBeanData.noofemployees.toString()
                  : "",
            },
            {
              'apiKeyValue': 'annualrevenue',
              'key': 'number',
              'type': 'input',
              'label': 'Annual Revenue',
              'required': true,
              'value': accountListBeanData.annualrevenue.toString() != "null"
                  ? accountListBeanData.annualrevenue.toString()
                  : "",
            },
            {
              'apiKeyValue': 'email',
              'key': 'email',
              'type': 'email',
              'label': 'Email',
              'placeholder': "Enter Your Username",
              'required': 'True',
              'value': accountListBeanData.email.toString() != "null"
                  ? accountListBeanData.email.toString()
                  : "",
            },
            {
              'apiKeyValue': 'phone',
              'key': 'number',
              'type': 'input',
              'label': 'Phone Number',
              'required': true,
              'value': accountListBeanData.phone.toString() != "null"
                  ? accountListBeanData.phone.toString()
                  : "",
            },
            {
              'apiKeyValue': 'website',
              'key': 'url',
              'type': 'input',
              'label': 'Website',
              'required': true,
              'value': accountListBeanData.website.toString() != "null"
                  ? accountListBeanData.website.toString()
                  : "",
            },
            {
              'apiKeyValue': 'area',
              'key': 'text',
              'type': 'input',
              'label': 'Area',
              'required': true,
              'value': accountListBeanData.area.toString() != "null"
                  ? accountListBeanData.area.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address1',
              'key': 'text',
              'type': 'input',
              'label': 'Address 1',
              'required': true,
              'value': accountListBeanData.address1.toString() != "null"
                  ? accountListBeanData.address1.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address2',
              'key': 'text',
              'type': 'input',
              'label': 'Address 2',
              'required': false,
              'value': accountListBeanData.address2.toString() != "null"
                  ? accountListBeanData.address2.toString()
                  : "",
            },
            {
              'apiKeyValue': 'pincode',
              'key': 'number',
              'type': 'input',
              'label': 'Pincode ',
              'required': true,
              'value': accountListBeanData.pincode.toString() != "null"
                  ? accountListBeanData.pincode.toString()
                  : "",
            },
            {
              'apiKeyValue': 'state',
              'key': 'text',
              'type': 'input',
              'label': 'State ',
              'required': true,
              'value': accountListBeanData.state.toString() != "null"
                  ? accountListBeanData.state.toString()
                  : "",
            },
            {
              'apiKeyValue': 'country',
              'key': 'text',
              'type': 'input',
              'label': 'Country',
              'required': true,
              'value': accountListBeanData.country.toString() != "null"
                  ? accountListBeanData.country.toString()
                  : "",
            },
          ]
        });
      } else {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'owner_id',
              'key': 'number',
              'type': 'input',
              'label': 'Owner ID',
              'required': true,
              'readOnly': true,
              'value': ownerId ?? "",
              'visibility': false,
            },
            {
              'apiKeyValue': 'companyname',
              'key': 'text',
              'type': 'input',
              'label': 'Company Name',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'value': firstName ?? null,
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'industry',
              'key': 'Industry',
              'type': 'select',
              'label': 'Industry',
              'required': true,
              'items': industry != null
                  ? industry
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'noofemployees',
              'key': 'number',
              'type': 'input',
              'label': 'No of Employees',
              'required': true,
              'hint': 'Select a status',
              'value': "",
            },
            {
              'apiKeyValue': 'annualrevenue',
              'key': 'number',
              'type': 'input',
              'label': 'Annual Revenue',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'email',
              'key': 'email',
              'type': 'email',
              'label': 'Email',
              'placeholder': "Enter Your Username",
              'required': 'True',
              'value': "",
            },
            {
              'apiKeyValue': 'phone',
              'key': 'number',
              'type': 'input',
              'label': 'Phone Number',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'website',
              'key': 'url',
              'type': 'input',
              'label': 'Website',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'area',
              'key': 'text',
              'type': 'input',
              'label': 'Area',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'address1',
              'key': 'text',
              'type': 'input',
              'label': 'Address 1',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'address2',
              'key': 'text',
              'type': 'input',
              'label': 'Address 2',
              'required': false,
              'value': "",
            },
            {
              'apiKeyValue': 'pincode',
              'key': 'number',
              'type': 'input',
              'label': 'Pincode ',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'state',
              'key': 'text',
              'type': 'input',
              'label': 'State ',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'country',
              'key': 'text',
              'type': 'input',
              'label': 'Country',
              'required': true,
              'placeholder': 'Country',
              'helperText': "test",
            },
          ]
        });
      }

      setState(() {
        isLoader = false;
        isLoading = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> addAccountNewAPI(String jsonString) async {
    String urlController = ServerUrl.crmCreateAccount;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonString.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> taskAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmTaskViewDetails;
    var objToSend = {
      "\"condition\"": [
        {"\"accountId\"": id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (taskDetailsResponseBean.status) {
      taskListBeanData = taskDetailsResponseBean.data;

      taskListBeanData = taskDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> notesAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmNotesView;
    var objToSend = {
      "\"condition\"": [
        {"\"accountId\"": id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    NotesDetailsBean notesDetailsBeanResponse = NotesDetailsBean.fromJson(map);
    if (notesDetailsBeanResponse.status) {
      notesDetailsBean = notesDetailsBeanResponse;
      notesListDetailsBean = notesDetailsBeanResponse.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onTaskAPICall() async {
    isLoader = false;

    setState(() {
      taskAPI();
      this.mounted;
    });
  }

  Future<void> onNotesAPICall() async {
    isLoader = false;

    setState(() {
      notesAPI();
      this.mounted;
    });
  }
}
