import 'dart:convert';

import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/ConditionalDropDown/awesome_dropdown.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/deals_details_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/leads_details_bean.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';

import '../../main.dart';

class AddCreateTaskScreen extends StatefulWidget {
  Color primaryColor;
  Color secondaryColor;
  Function() onCallBack;

  AddCreateTaskScreen({
    Key key,
    this.primaryColor,
    this.secondaryColor,
    this.onCallBack,
  }) : super(key: key);

  @override
  _AddCreateTaskScreenState createState() => _AddCreateTaskScreenState();
}

class _AddCreateTaskScreenState extends State<AddCreateTaskScreen> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();
  bool _isBackPressedOrTouchedOutSide = false,
      _isDropDownOpened = false,
      _isPanDown = false;
  String _contactName = '';
  String _leadName = '';
  String _activityTypeName = '';
  String _statusName = '';

  String _accountId = '';
  String _contactId = '';
  String _activityTypeId = '';
  String _leadId = '';
  String _statusId = '';

  DateTime selectedDate = DateTime.now();
  String selectedDateString;
  String ownerId;

  bool isSelectedDd = false;
  bool isLoader = true;

  List<String> contact = new List.empty(growable: true);
  List<String> account = new List.empty(growable: true);
  List<String> status = new List.empty(growable: true);

  List<String> lead = new List.empty(growable: true);
  List<String> activityType = new List.empty(growable: true);
  List<String> deal = new List.empty(growable: true);
  List<ContactListBean> contactListBean = new List.empty(growable: true);
  List<LeadsListBean> leadListBean = new List.empty(growable: true);
  List<MasterDataList> masterDataList = new List.empty(growable: true);

  final descController = TextEditingController();
  final dateController = TextEditingController();

  var _formKey = GlobalKey<FormState>();

  @override
  void initState() {
    _contactName = 'Select Item';
    _leadName = 'Select Item';
    _activityTypeName = 'Select Item';
    _statusName = 'Select Item';

    dateController.text =
        MyApp.buiFormatter.dateToStringDateReverseWithOutTime(selectedDate);

    callContactAPI();
    initSharedPref();
    super.initState();
  }

  @override
  void dispose() {
    descController.dispose();
    dateController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: _removeFocus,
      onPanDown: (focus) {
        _isPanDown = true;
        _removeFocus();
      },
      child: Scaffold(
        backgroundColor: Colors.white,
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text(
            AppStrings.CREATE_TASK,
            style: TextStyle(
                fontSize: FontSize.FONT_18,
                fontFamily: FontFamily.Gilroy,
                fontWeight: FontWeight.bold,
                color: AppColors.appWhite),
          ),
          backgroundColor: widget.primaryColor,
        ),
        body: isLoader
            ? isLoaderView
            : Form(
                key: _formKey,
                child: SingleChildScrollView(
                  child: Column(
                    children: [
                      _buildTextView("Contact"),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 16, right: 20),
                        child: AwesomeDropDown(
                          isPanDown: _isPanDown,
                          dropDownList: contact,
                          isBackPressedOrTouchedOutSide:
                              _isBackPressedOrTouchedOutSide,
                          selectedItem: _contactName,
                          onDropDownItemClick: (selectedItem) {
                            _contactName = selectedItem;
                            setState(() {
                              isSelectedDd = true;
                            });

                            for (int i = 0; i < contactListBean.length; i++) {
                              if (contactListBean[i].firstname +
                                      " " +
                                      contactListBean[i].secondName ==
                                  selectedItem) {
                                account = [
                                  if (contactListBean[i].accountid != null)
                                    contactListBean[i]
                                        .accountid
                                        .companyname
                                        .toString()
                                ];

                                _contactId = contactListBean[i].id.toString();
                                _accountId =
                                    contactListBean[i].accountid.id.toString();
                                break;
                              }
                            }
                          },
                          dropStateChanged: (isOpened) {
                            _isDropDownOpened = isOpened;
                            if (!isOpened) {
                              _isBackPressedOrTouchedOutSide = false;
                            }
                          },
                        ),
                      ),
                      Visibility(
                        visible: isSelectedDd,
                        child: _buildTextView("Account"),
                      ),
                      Visibility(
                        visible: isSelectedDd,
                        child: Container(
                          width: MediaQuery.of(context).size.width,
                          margin: EdgeInsets.only(
                              top: 5, bottom: 2, left: 16, right: 20),
                          child: AwesomeDropDown(
                            dropDownListTextStyle: TextStyle(
                                fontFamily: FontFamily.Gilroy,
                                fontSize: FontSize.FONT_18),
                            dropDownList: account,
                            onDropDownItemClick: (selectedItem) {
                              for (int i = 0; i < contactListBean.length; i++) {
                                if (contactListBean[i].accountid.companyname ==
                                    selectedItem) {
                                  break;
                                }
                              }
                            },
                          ),
                        ),
                      ),
                      Visibility(
                        visible: isSelectedDd,
                        child: _buildTextView("Deal"),
                      ),
                      Visibility(
                        visible: isSelectedDd,
                        child: Container(
                          width: MediaQuery.of(context).size.width,
                          margin: EdgeInsets.only(
                              top: 5, bottom: 2, left: 16, right: 20),
                          child: AwesomeDropDown(
                            dropDownListTextStyle: TextStyle(
                                fontFamily: FontFamily.Gilroy,
                                fontSize: FontSize.FONT_18),
                            dropDownList: account,
                            onDropDownItemClick: (selectedItem) {
                              for (int i = 0; i < contactListBean.length; i++) {
                                if (contactListBean[i].accountid.companyname ==
                                    selectedItem) {
                                  break;
                                }
                              }
                            },
                          ),
                        ),
                      ),
                      _buildTextView("Description"),
                      Container(
                        padding: const EdgeInsets.all(8),
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 8, right: 8),
                        child: TextFormField(
                          controller: descController,
                          decoration: new InputDecoration(
                            filled: true,
                            fillColor: Colors.white,
                            hintText: "Description",
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                              borderSide: BorderSide(
                                color: widget.primaryColor,
                                width: 1.0,
                              ),
                            ),
                            focusedBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                              borderSide: BorderSide(
                                color: widget.primaryColor,
                              ),
                            ),
                            contentPadding: const EdgeInsets.all(5.0),
                            border: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                            ),
                          ),
                          validator: (value) {
                            if (value.length == 0) {
                              return "Please Enter Some Text";
                            }
                            return null;
                          },
                          onChanged: (String value) {},
                          onFieldSubmitted: (value) {
                            Utils.hideKeyboard(context);
                          },
                        ),
                      ),
                      _buildTextView("Lead"),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 16, right: 20),
                        child: AwesomeDropDown(
                          isPanDown: _isPanDown,
                          dropDownList: lead,
                          isBackPressedOrTouchedOutSide:
                              _isBackPressedOrTouchedOutSide,
                          selectedItem: _leadName,
                          onDropDownItemClick: (selectedItem) {
                            _leadName = selectedItem;

                            for (int i = 0; i < leadListBean.length; i++) {
                              if (leadListBean[i].firstname +
                                      " " +
                                      leadListBean[i].lastname ==
                                  selectedItem) {
                                _leadId = leadListBean[i].id.toString();

                                break;
                              }
                            }
                          },
                          dropStateChanged: (isOpened) {
                            _isDropDownOpened = isOpened;
                            if (!isOpened) {
                              _isBackPressedOrTouchedOutSide = false;
                            }
                          },
                        ),
                      ),
                      _buildTextView("Activity Type"),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 16, right: 20),
                        child: AwesomeDropDown(
                          isPanDown: _isPanDown,
                          dropDownList: activityType,
                          isBackPressedOrTouchedOutSide:
                              _isBackPressedOrTouchedOutSide,
                          selectedItem: _activityTypeName,
                          onDropDownItemClick: (selectedItem) {
                            _activityTypeName = selectedItem;
                            for (int i = 0; i < masterDataList.length; i++) {
                              if (masterDataList[i].page == "Activity") {
                                if (masterDataList[i].key ==
                                    "Type") if (masterDataList[i]
                                        .name
                                        .toString() ==
                                    selectedItem) {
                                  _activityTypeId =
                                      masterDataList[i].id.toString();
                                  break;
                                }
                              }
                            }
                          },
                          dropStateChanged: (isOpened) {
                            _isDropDownOpened = isOpened;
                            if (!isOpened) {
                              _isBackPressedOrTouchedOutSide = false;
                            }
                          },
                        ),
                      ),
                      _buildTextView("Due Date"),
                      Container(
                        padding: const EdgeInsets.all(8),
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 8, right: 8),
                        child: TextFormField(
                          controller: dateController,
                          onTap: () {
                            // Below line stops keyboard from appearing
                            Utils.hideKeyboard(context);
                            _selectDate(context);
                            // Show Date Picker Here
                          },
                          decoration: new InputDecoration(
                            filled: true,
                            fillColor: Colors.white,
                            hintText: "Description",
                            enabledBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                              borderSide: BorderSide(
                                color: widget.primaryColor,
                                width: 1.0,
                              ),
                            ),
                            focusedBorder: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                              borderSide: BorderSide(
                                color: widget.primaryColor,
                              ),
                            ),
                            contentPadding: const EdgeInsets.all(5.0),
                            border: OutlineInputBorder(
                              borderRadius: BorderRadius.circular(5.0),
                            ),
                          ),
                          validator: (value) {
                            if (value.length == 0) {
                              return "Please Enter Some Text";
                            }
                            return null;
                          },
                          onChanged: (String value) {},
                        ),
                      ),
                      _buildTextView("Status"),
                      Container(
                        width: MediaQuery.of(context).size.width,
                        margin: EdgeInsets.only(
                            top: 5, bottom: 2, left: 16, right: 20),
                        child: AwesomeDropDown(
                          isPanDown: _isPanDown,
                          dropDownList: status,
                          isBackPressedOrTouchedOutSide:
                              _isBackPressedOrTouchedOutSide,
                          selectedItem: _statusName,
                          onDropDownItemClick: (selectedItem) {
                            _statusName = selectedItem;
                            for (int i = 0; i < masterDataList.length; i++) {
                              if (masterDataList[i].page == "Activity") {
                                if (masterDataList[i].key ==
                                    "Status") if (masterDataList[i]
                                        .name
                                        .toString() ==
                                    selectedItem) {
                                  _statusId = masterDataList[i].id.toString();
                                  break;
                                }
                              }
                            }
                          },
                          dropStateChanged: (isOpened) {
                            _isDropDownOpened = isOpened;
                            if (!isOpened) {
                              _isBackPressedOrTouchedOutSide = false;
                            }
                          },
                        ),
                      ),
                      Container(
                          margin: EdgeInsets.only(
                              top: 5, bottom: 2, left: 16, right: 20),
                          child: Row(
                            mainAxisAlignment: MainAxisAlignment.spaceAround,
                            children: [
                              Button.mini(
                                width: MediaQuery.of(context).size.width * 0.3,
                                onPressed: () {
                                  Navigator.of(context).pop();
                                },
                                buttonSize: ButtonSize.medium,
                                buttonType: ButtonType.custom,
                                btnColor: AppColors.APP_BUTTON_GREY,
                                btnTextColor: widget.primaryColor,
                                btnText: AppStrings.CANCEL,
                                elevation: 10,
                              ),
                              Button.mini(
                                width: MediaQuery.of(context).size.width * 0.3,
                                onPressed: () {
                                  final isValid =
                                      _formKey.currentState.validate();
                                  if (!isValid) {
                                    return;
                                  } else {
                                    if (_accountId.isNotEmpty &&
                                        _contactId.isNotEmpty &&
                                        dateController.text.isNotEmpty &&
                                        _leadId.isNotEmpty &&
                                        _activityTypeId.isNotEmpty &&
                                        _statusId.isNotEmpty) {
                                      callTaskAddedAPI();
                                    } else {
                                      Toast.show(
                                          "Required All Fields", context);
                                    }
                                  }
                                  _formKey.currentState.save();
                                },
                                buttonSize: ButtonSize.medium,
                                buttonType: ButtonType.custom,
                                btnColor: widget.secondaryColor ??
                                    AppColors.APP_COLOR_PRIMARY,
                                btnTextColor: Colors.white,
                                btnText: AppStrings.SAVE,
                                elevation: 10,
                              ),
                            ],
                          ))
                    ],
                  ),
                ),
              ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                Padding(
                  padding: const EdgeInsets.all(8.0),
                  child: FlutterShimmnerLoadingWidget(
                    count: 1,
                    animate: true,
                    color: Colors.grey[200],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget _buildTextView(String label) {
    return Container(
        width: MediaQuery.of(context).size.width,
        margin: EdgeInsets.only(top: 5, left: 16, right: 20),
        child: RichText(
          text: TextSpan(
            text: label,
            style: TextStyle(
                fontSize: FontSize.FONT_18,
                fontFamily: FontFamily.Gilroy,
                fontWeight: FontWeight.bold,
                color: widget.primaryColor ?? AppColors.primaryColor),
            children: const <TextSpan>[
              TextSpan(
                  text: ' *',
                  style: TextStyle(
                      fontWeight: FontWeight.bold,
                      color: AppColors.errorWarning)),
            ],
          ),
        )

        /*Text(
        label,
        style: TextStyle(
            fontSize: FontSize.FONT_18,
            fontFamily: FontFamily.Gilroy,
            fontWeight: FontWeight.bold,
            color: widget.primaryColor ?? AppColors.primaryColor),
      ),*/
        );
  }

  /// this func is used to close dropDown (if open) when you tap or pandown anywhere in the screen
  void _removeFocus() {
    if (_isDropDownOpened) {
      setState(() {
        _isBackPressedOrTouchedOutSide = true;
      });
    }
  }

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  Future<void> callContactAPI() async {
    contact.clear();
    setState(() {
      isLoader = true;
    });

    String urlController = ServerUrl.crmContactViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      contactListBean = contactDetailsResponseBean.data;
      contact = [
        for (int i = 0; i < contactDetailsResponseBean.data.length; i++)
          contactDetailsResponseBean.data[i].firstname +
              " " +
              contactDetailsResponseBean.data[i].secondName,
      ];

      leadScreenViewDetailsAPI();
    }
  }

  Future<void> leadScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmLeadViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    LeadsDetailsBean leadDetailsResponseBean = LeadsDetailsBean.fromJson(map);

    if (leadDetailsResponseBean.status) {
      leadListBean = leadDetailsResponseBean.data;
      lead = [
        for (int i = 0; i < leadDetailsResponseBean.data.length; i++)
          leadDetailsResponseBean.data[i].firstname +
              " " +
              leadDetailsResponseBean.data[i].lastname,
      ];
      setState(() {
        isLoader = false;
      });
      dealScreenViewDetailsAPI();
    }
  }

  Future<void> dealScreenViewDetailsAPI({String obj}) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmDealsViewDetails;
    var objToSend;
    if (obj == null) {
      objToSend = {
        "\"condition\"": [],
        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
      };
    }
    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DealsDetailsBean dealsDetailsBean = DealsDetailsBean.fromJson(map);

    if (dealsDetailsBean.status) {
      deal = [
        for (int i = 0; i < dealsDetailsBean.data.length; i++)
          dealsDetailsBean.data[i].companyid.companyname.toString(),
      ];

      setState(() {
        isLoader = false;
      });
      taskMasterViewAPI();
    }
  }

  Future<void> taskMasterViewAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmMasterDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {}
    if (detailsBean.status) {
      masterDataList = detailsBean.data;
      activityType = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Type") detailsBean.data[i].name,
      ];
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Status") detailsBean.data[i].name,
      ];
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callTaskAddedAPI() async {
    var description = descController.text;

    if (ownerId == null) {
      ownerId = "13";
    }
    var objToSend = {
      "\"owner_id\"": "\"$ownerId\"",
      "\"accountid\"": "\"$_accountId\"",
      "\"contactid\"": "\"$_contactId\"",
      "\"desc\"": "\"$description\"",
      "\"leadid\"": "\"$_leadId\"",
      "\"activitytypeid\"": "\"$_activityTypeId\"",
      "\"dealid\"": "\"$_accountId\"",
      "\"duedate\"": "\"${dateController.text}\"",
      "\"statusid\"": "\"$_statusId\"",
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String urlController = ServerUrl.crmCreateTask;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: objToSend.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> _selectDate(BuildContext context) async {
    final DateTime picked = await showDatePicker(
      context: context,
      initialDate: selectedDate,
      firstDate: DateTime(2000, 8),
      lastDate: DateTime(2101),
      builder: (BuildContext context, Widget child) {
        return Theme(
          data: Theme.of(context).copyWith(
              colorScheme: Theme.of(context).colorScheme.copyWith(
                  primary:
                      widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY)),
          child: child,
        );
      },
    );
    if (picked != null && picked != selectedDate)
      setState(() {
        selectedDate = picked;
        selectedDateString =
            MyApp.buiFormatter.dateTimeToStringDate(selectedDate);
        dateController.text = selectedDateString;
      });
  }
}
