import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Screens/CRM/ContactViewDetailScreen.dart';
import 'package:example/Screens/CRM/FilterScreen.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/Screens/platform.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import 'DrawerMenuHeaderDesign.dart';

class ContactScreen extends DrawerContent {
  ContactScreen({Key key, this.title, this.permissionStr});

  final String title;
  int permissionStr;

  @override
  _ContactScreenState createState() => _ContactScreenState();
}

class _ContactScreenState extends State<ContactScreen> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  ContactDetailsBean contactDetailsBean;

  List<ContactListBean> contactListBeanData = new List.empty(growable: true);

  bool isLoader = true;

  String jsonFormFilterData;

  List<String> actions = ["Edit", "Delete"];

  List<dynamic> status = new List.empty(growable: true);
  List<dynamic> industry = new List.empty(growable: true);
  List<dynamic> leadSource = new List.empty(growable: true);
  List<dynamic> sales = new List.empty(growable: true);
  String dateTime;

  bool isLoading = false;

  ScrollController _scrollController = new ScrollController();
  int startIndex = 0;
  int offset = 10;

  bool visible = true;

  @override
  void initState() {
    contactScreenViewDetailsAPI();
    colorCodePref();
    //widget.permissionStr = int.parse("10000", radix: 2);
    permissionMethod();
    _scrollController.addListener(() {
      if (_scrollController.position.pixels ==
          _scrollController.position.maxScrollExtent) {
        startIndex = startIndex + offset;
        contactScreenViewDetailsAPI();
      }
    });
    super.initState();
  }

  var now = DateTime.now();

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: true,
                isFilterClicked: onFilterClicked,
              ),
              RefreshIndicator(
                color: theme,
                onRefresh: () async {
                  startIndex = 0;
                  contactListBeanData =
                      List<ContactListBean>.empty(growable: true);
                  return await contactScreenViewDetailsAPI();
                },
                child: isLoader
                    ? Stack(
                        children: [
                          deleteDetailsView,
                          isLoaderView,
                        ],
                      )
                    : Stack(
                        children: [deleteDetailsView, detailsView],
                      ),
              )
            ],
          ),
        ),
        floatingActionButton: Visibility(
          visible: visible,
          child: FloatingActionButton(
            mini: true,
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child: ContactViewDetailScreen(
                        dateTime: dateTime,
                        pageTitle: "Contact Info",
                        type: "Add",
                        theme: theme,
                        onCallBack: onCallBackAPI,
                        secondaryColor: secondaryColor,
                      ),
                      transitionEffect: TransitionEffect.SCALE));
            },
            child: const Icon(Icons.add),
            backgroundColor: theme,
          ),
        ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
          AppStrings.delete_and_edit,


          style: MyApp.textTheme.bodyText1.copyWith(  color: Colors.black87,
              fontSize: FontSize.FONT_12,
              //fontStyle: FontStyle.italic,
              fontWeight: FontWeight.bold,
              letterSpacing: 1,
              wordSpacing: 1
          )
      ),
      color: Colors.grey[400],
    );
  }

  Widget get addDetailsView {
    return Container(
      height: 60,
      padding: EdgeInsets.all(5),
      alignment: Alignment.center,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Container(
            height: 50,
            width: 50,
            padding: EdgeInsets.all(5),
            child: ClipOval(
              child: Material(
                color: theme, // button color
                child: InkWell(
                  splashColor: Colors.white, // inkwell color
                  child: SizedBox(
                      width: 56,
                      height: 56,
                      child: Icon(Icons.add, color: Colors.white)),
                  onTap: () {
                    contactScreenViewDetailsAPI();
                  },
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget get detailsView {
    if (contactListBeanData.length != 0) {
      return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: contactListBeanData.length +
              1, // Add one more item for progress indicator
          padding: EdgeInsets.symmetric(vertical: 8.0),
          itemBuilder: (BuildContext context, int index) {
            if (index == contactListBeanData.length) {
              return _buildProgressIndicator();
            } else {
              return buildUsageItems(context, index);
            }
          },
          controller: _scrollController,
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Widget _buildProgressIndicator() {
    return new Padding(
      padding: const EdgeInsets.all(8.0),
      child: new Center(
        child: new Opacity(
          opacity: isLoading ? 1.0 : 00,
          //child: new CircularProgressIndicator(),
          child: Loader(
            loaderColor: theme,
          ),
        ),
      ),
    );
  }

  Widget buildUsageItems(BuildContext context, int index) {
    return Dismissible(
      key: Key(contactListBeanData[index].id.toString()),
      background: slideRightBackground(),
      secondaryBackground: slideLeftBackground(),
      confirmDismiss: (direction) async {
        if (direction == DismissDirection.endToStart) {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this ${contactListBeanData[index].firstname}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  callContactsDeleteAPI(
                      contactListBeanData[index].id.toString());
                  contactListBeanData.removeAt(index);
                });
              },
            );
            return res;
          } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
            Toast.show("viewAccessForOwn", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForOwn) {
            Toast.show("viewAndAddAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            Toast.show("fullAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
            Toast.show("viewAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForAllUser) {
            Toast.show("viewAndAddAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr == AppStrings.noAccessPermission) {
            Toast.show("noAccessPermission", context);
            return false;
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        } else {
          return Navigator.of(context).push(MaterialPageRoute(
            builder: (context) {
              return ContactViewDetailScreen(
                dateTime: dateTime,
                pageTitle: "Contact Info",
                contactListDetails: contactListBeanData[index],
                theme: theme,
                onCallBack: onCallBackAPI,
                id: contactListBeanData[index].id.toString(),
                secondaryColor: secondaryColor,
                companyName: contactListBeanData[index].accountid.companyname,
                permissionStr: widget.permissionStr,
                visible: visible,
              );
            },
          ));
        }
      },
      child: InkWell(
        onTap: () {
          Navigator.push(
              context,
              Transition(
                  child: ContactViewDetailScreen(
                    dateTime: dateTime,
                    pageTitle: "Contact Info",
                    contactListDetails: contactListBeanData[index],
                    theme: theme,
                    onCallBack: onCallBackAPI,
                    id: contactListBeanData[index].id.toString(),
                    secondaryColor: secondaryColor,
                    companyName:
                        contactListBeanData[index].accountid.companyname,
                    permissionStr: widget.permissionStr,
                    visible: visible,
                  ),
                  transitionEffect: TransitionEffect.SCALE));
        },
        child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: contactListBeanData[index].firstname ?? "",
          cardName: contactListBeanData[index].accountid.companyname ?? "",
          statusTag: contactListBeanData[index].statusid.name ?? "",
          dashBoardCardTime: dateTime == "dd-mm-yyyy" && dateTime != null
              ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(contactListBeanData[index].createdAt)}" ??
                  ""
              : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(contactListBeanData[index].createdAt)}" ??
                  "",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  Future<void> callContactsDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmDeleteContact;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      contactScreenViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    String dateTme = prefs.getString(AppStrings.date);

    dateTime = dateTme;
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  Future<void> onAPICallBack() async {
    isLoader = false;

    setState(() {
      contactScreenViewDetailsAPI();
      this.mounted;
    });
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Message Pressed");
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Setting Pressed");
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Notification Pressed");
    });
  }

  Future<void> onPopUpMenuPressed(int count) async {
    switch (count) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
      context,
      Transition(
          child: ThemePickerScreen(
            onBackPressed: onBackPressed,
          ),
          transitionEffect: TransitionEffect.SCALE),
    );
  }

  Future<void> onCallBackAPI() async {
    isLoader = false;

    setState(() {
      contactScreenViewDetailsAPI();
      this.mounted;
    });
  }

  Future<void> onFilterClicked() async {
    colorCodePref();
    setState(() {
      Navigator.push(
          context,
          Transition(
              child: FilterScreen(
                primaryColor: theme,
                secondaryColor: secondaryColor,
                filterFormData: jsonFormFilterData,
                fromPageId: "contactId",
                isActionApply: (String obj) {
                  callFilterAPI(obj);
                },
              ),
              transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
    });
  }

  Future<void> callFilterAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmContactViewDetails;
    var objToSend = {
      "\"condition\"": [obj],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (contactDetailsResponseBean.status) {
      contactDetailsBean = contactDetailsResponseBean;
      contactListBeanData = contactDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> contactScreenViewDetailsAPI() async {
    setState(() {
      if (startIndex == 0)
        isLoader = true;
      else
        isLoading = true;
    });
    String urlController = ServerUrl.crmContactViewDetails;
    var objToSend = {
      "\"condition\"": [],
      /* "\"page\"": {
        "\"orderby\"": [
          {
            "\"id\"": "\"${"asc"}\"",
          }
        ],
        "\"no\"": "\"${startIndex.toString()}\"",
        "\"size\"": "\"${10}\"",
      },*/
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (startIndex == 0 && contactDetailsResponseBean.data == null) {
      setState(() {
        isLoader = true;
      });
    }

    if (contactDetailsResponseBean.status) {
      contactDetailsBean = contactDetailsResponseBean;
      contactListBeanData = contactDetailsResponseBean.data;
      status = [
        for (int i = 0; i < contactListBeanData.length; i++)
          {
            'label': contactListBeanData[i].statusid.name,
            'value': contactListBeanData[i].statusid.id.toString(),
          },
      ];

      status.insert(0, {
        'label': '',
        'value': '',
      });

      final jsonList = status.map((item) => jsonEncode(item)).toList();

      final uniqueJsonList = jsonList.toSet().toList();

      status = uniqueJsonList.map((item) => jsonDecode(item)).toList();

      jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'statusid',
            'key': 'select1',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'value': '',
            'items': status ??
                [
                  {
                    'label': "Select",
                    'value': "",
                  },
                  {
                    'value': "1",
                    'label': "New",
                  },
                  {
                    'value': "2",
                    'label': "Active",
                  },
                  {
                    'value': "3",
                    'label': "Inactive",
                  },
                  {
                    'value': "4",
                    'label': "Hot",
                  },
                ]
          },
        ]
      });

      setState(() {
        if (startIndex == 0)
          isLoader = false;
        else
          isLoading = false;
      });
    }
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }
}
