import 'dart:convert';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/logger.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class CreateNoteScreen extends StatefulWidget {
  String id;
  Color primaryColor;
  Color secondaryColor;
  Function() onCallBack;
  String fromPageId;

  CreateNoteScreen(
      {Key key,
      this.id,
      this.primaryColor,
      this.secondaryColor,
      this.onCallBack,
      this.fromPageId})
      : super(key: key);

  @override
  _CreateNoteScreenState createState() => _CreateNoteScreenState();
}

class _CreateNoteScreenState extends State<CreateNoteScreen> {
  String form;
  dynamic formGeneral;
  dynamic response;
  bool isLoading = true;
  Map decorations;

  Map validations = {'text1': validationExample};
  Map errorText = {
    'input1': "Error Value",
    'text1': "Enter the Password",
  };
  List<dynamic> activityType = new List.empty(growable: true);
  List<dynamic> status = new List.empty(growable: true);
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();
    initSharedPref();
  }

  String ownerId;

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
    getFormDetails();
  }

  Future<void> getFormDetails() async {
    setState(() {
      isLoading = true;
    });

    form = json.encode({
      'fields': [
        {
          'apiKeyValue': widget.fromPageId ?? 'accountId',
          'key': 'Company Name',
          'type': 'input',
          'label': 'Account Id',
          'required': true,
          'readOnly': true,
          'value': widget.id != null ? widget.id : "",
          'visibility': false,
        },
        {
          'apiKeyValue': 'owner_id',
          'key': 'Company Name',
          'type': 'input',
          'label': 'Owner ID',
          'required': true,
          'readOnly': true,
          'value': ownerId ?? "",
          'visibility': false,
        },
        {
          'apiKeyValue': 'desc',
          'key': 'Desc',
          'type': 'textInput',
          'label': 'Description',
          'required': true,
          'readOnly': false,
          'value': "",
        },
      ]
    });
    setState(() {
      isLoading = false;
    });

    decorations = {
      'Company Name': InputDecoration(
        fillColor: AppColors.appWhite,
        prefixIcon: Icon(
          Icons.email,
          color: widget.primaryColor ?? AppColors.APP_COLOR_PRIMARY,
        ),
        focusedBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
              color: widget.primaryColor ?? AppColors.halfWhite, width: 2),
        ),
        enabledBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.halfWhite,
            width: 2.0,
          ),
        ),
        errorBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
        border: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
      ),
      'Desc': InputDecoration(
        filled: true,
        fillColor: AppColors.APP_COLOR_WHITE,
        focusedBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
              color: widget.primaryColor ?? AppColors.halfWhite, width: 2),
        ),
        enabledBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.halfWhite,
            width: 2.0,
          ),
        ),
        errorBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
        border: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
      ),
      'activitytypeid': InputDecoration(
        focusedBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
              color: widget.primaryColor ?? AppColors.halfWhite, width: 2),
        ),
        enabledBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.halfWhite,
            width: 2.0,
          ),
        ),
        errorBorder: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
        border: OutlineInputBorder(
          borderRadius: BorderRadius.circular(7.0),
          borderSide: BorderSide(
            color: AppColors.errorWarning,
            width: 2.0,
          ),
        ),
      ),
    };
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
            title: Text(AppStrings.CREATE_NOTES),
            backgroundColor: widget.primaryColor,
          ),
          body: isLoading
              ? Center(
                  child: Loader(
                  loaderColor: widget.primaryColor,
                ))
              : _formView),
    );
  }

  Widget get _formView {
    return new ListView(
      children: [
        Form(
          child: new Center(
            child: new Column(children: <Widget>[
              new JsonSchema(
                primaryColor: widget.primaryColor ?? AppColors.primaryColor,
                secondaryColor:
                    widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                decorations: decorations,
                form: form,
                validations: validations,
                onChanged: (dynamic response) {
                  this.response = response;
                },
                buttonText: "Create Notes" ?? AppStrings.ADD,
                actionSave: (data) {
                  setState(() {
                    formGeneral = data;
                  });
                  Utils.hideKeyboard(context);
                  var jsonObj = {
                    for (int i = 0; i < data['fields'].length; i++)
                      json.encode(data['fields'][i]['apiKeyValue'].toString()):
                          json.encode(
                              data['fields'][i]['value'].toString().trim()),
                    "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                    "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                    "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                  };
                  Logger.logMessage(Log.DEBUG, jsonObj);

                  createNotesAPI(jsonObj.toString());
                },
                actionCancel: () {
                  Navigator.pop(context);
                },
                buttonSave: true,
              ),
            ]),
          ),
        ),
      ],
    );
  }

  static String validationExample(field, value) {
    String pattern =
        r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{8,}$';
    RegExp regExp = new RegExp(pattern);

    if (value.isEmpty) {
      return 'Please enter some text';
    }
    if (value.length < 8) {
      return 'Password is too short';
    }

    if (!regExp.hasMatch(value)) {
      return 'Password contains Minimum 1 Upper case Minimum 1 lowercase Minimum 1 Numeric Number Minimum 1 Special Character Common Allow Character ( ! @ # \$ & * ~ )';
    }
    return null;
  }

  Future<void> createNotesAPI(String jsonString) async {
    setState(() {
      isLoading = true;
    });
    String urlController = ServerUrl.crmActivitiesCreateNotes;

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonString);
    Map map = jsonDecode(res);

    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      setState(() {
        isLoading = false;
      });
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    }
  }
}
