import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_loading_widget.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/deal_bean.dart';
import 'package:example/Models/deals_details_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import '../../main.dart';
import '../NoDataFoundScreen.dart';
import '../platform.dart';
import 'DealScreenViewDetails.dart';

class DealScreen extends DrawerContent {
  DealScreen({Key key, this.permissionStr});

  int permissionStr;

  @override
  DealScreenState createState() => DealScreenState();
}

class DealScreenState extends State<DealScreen> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  bool isLoader = true;

  String jsonFormFilterData;

  List<dynamic> companyName = List<dynamic>.empty(growable: true);

  List<dynamic> contactName = List<dynamic>.empty(growable: true);

  String dateTime;
  String currency;
  var now = DateTime.now();

  bool isLoading = false;

  ScrollController _scrollController = new ScrollController();
  int startIndex = 0;
  int offset = 10;

  bool visible = true;

  DealsDetailsBean dealsBean;

  List<DealsListDetailsBean> dealsListBean = new List<DealsListDetailsBean>.empty(growable: true);

  List<ContactListBean> contactListBeanData = new List<ContactListBean>.empty(growable: true);

  String ownerId;

  @override
  void initState() {
    colorCodePref();

    //widget.permissionStr = int.parse("10000", radix: 2);

    permissionMethod();
    dealViewDetailsAPI();


    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Column(
          children: [
            DrawerMenuDesign(
              onMenuClicked: widget.onMenuPressed,
              isFilterEnabled: true,
              isFilterClicked: () {},
            ),
            isLoader
                ? Stack(
                    children: [
                      deleteDetailsView,
                      isLoaderView,
                    ],
                  )
                : Stack(
                    children: [deleteDetailsView, detailsView],
                  ),
          ],
        ),

        floatingActionButton: Visibility(
          visible: true ?? visible,
          child: FloatingActionButton(
            mini: true,
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child:
                      DealScrenViewDetails(
                        pageTitle: "Info",
                        type: "Add",
                        dateTime: dateTime,
                        currency: currency,
                        onCallBack: dealViewDetailsAPI,
                        primaryColor: primaryColor,
                        secondaryColor: secondaryColor,
                        permissionStr: widget.permissionStr,
                        visible: visible,
                        ownerId: ownerId,
                      )

                      ,
                      transitionEffect: TransitionEffect.SCALE));
            },
            child: const Icon(Icons.add),
            backgroundColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
          AppStrings.delete_and_edit,


          style: MyApp.textTheme.bodyText1.copyWith(  color: Colors.black87,
              fontSize: FontSize.FONT_12,
              //fontStyle: FontStyle.italic,
              fontWeight: FontWeight.bold,
              letterSpacing: 1,
              wordSpacing: 1
          )
      ),
      color: Colors.grey[400],
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get detailsView {
    if (dealsListBean.length != 0) {
      return RefreshIndicator(
        color: primaryColor,
        onRefresh: () async {
          startIndex = 0;
          dealsListBean = List<DealsListDetailsBean>.empty(growable: true);
          return await dealViewDetailsAPI();
        },
        child: Container(
          height: MediaQuery.of(context).size.height * 0.80,
          margin: EdgeInsets.only(top: 30),
          child: ListView.builder(
            itemCount: dealsListBean.length +
                1, // Add one more item for progress indicator
            padding: EdgeInsets.symmetric(vertical: 8.0),
            itemBuilder: (BuildContext context, int index) {
              if (index == dealsListBean.length) {
                return _buildProgressIndicator();
              } else {
                return buildUsageItems(context, index);
              }
            },
            controller: _scrollController,
          ),
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Widget _buildProgressIndicator() {
    return new Padding(
      padding: const EdgeInsets.all(8.0),
      child: new Center(
        child: new Opacity(
          opacity: isLoading ? 1.0 : 00,
          //child: new CircularProgressIndicator(),
          child: Loader(
            loaderColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget buildUsageItems(BuildContext context, int index) {
    return Dismissible(
      key: Key(dealsListBean[index].id.toString()),
      background: slideRightBackground(),
      secondaryBackground: slideLeftBackground(),
      confirmDismiss: (direction) async {
        if (direction == DismissDirection.endToStart) {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this deal ${dealsListBean[index].name}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  callDealDeleteAPI(dealsListBean[index].id.toString());
                  dealsListBean.removeAt(index);
                });


              },
            );
            return res;
          } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
            Toast.show("viewAccessForOwn", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForOwn) {
            Toast.show("viewAndAddAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            Toast.show("fullAccessForOwn", context);
            return false;
          } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
            Toast.show("viewAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr ==
              AppStrings.viewAndAddAccessForAllUser) {
            Toast.show("viewAndAddAccessForAllUser", context);
            return false;
          } else if (widget.permissionStr == AppStrings.noAccessPermission) {
            Toast.show("noAccessPermission", context);
            return false;
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        } else {

          return Navigator.of(context).push(MaterialPageRoute(
            builder: (context) {
              return DealScrenViewDetails(
                pageTitle: "Info",
                dateTime: dateTime,
                currency: currency,
                id: dealsListBean[index].id,
                dealListBean: dealsListBean[index],
                onCallBack: dealViewDetailsAPI,
                primaryColor: primaryColor,
                secondaryColor: secondaryColor,
                permissionStr: widget.permissionStr,
                visible: visible,
                ownerId: ownerId,
              );
            },
          ));
        }
      },
      child: InkWell(
        onTap: () {
          Navigator.push(
              context,
              Transition(
                  child:  DealScrenViewDetails(
                    pageTitle: "Info",
                    dateTime: dateTime,
                    currency: currency,
                    id: dealsListBean[index].id,
                    dealListBean: dealsListBean[index],
                    onCallBack: dealViewDetailsAPI,
                    primaryColor: primaryColor,
                    secondaryColor: secondaryColor,
                    permissionStr: widget.permissionStr,
                    visible: visible,
                    ownerId: ownerId,
                  ),
                  transitionEffect: TransitionEffect.SCALE));
        },
        child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: dealsListBean[index].name ?? "",
          cardName: dealsListBean[index].companyid.companyname ?? "",
          statusTag: dealsListBean[index].statusid.name ?? "",
          dashBoardCardTime: dateTime == "dd-mm-yyyy" && dateTime != null
              ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(dealsListBean[index].closingdate)}" ??
              ""
              : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(dealsListBean[index].closingdate)}" ??
              "",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    String dateTme = prefs.getString(AppStrings.date);
    String currencySymbol = prefs.getString(AppStrings.currency);

    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });

    dateTime = dateTme;
    currency = currencySymbol;
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }

  Future<void> dealViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });

    String urlController = ServerUrl.crmDealsViewDetails;

    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DealsDetailsBean dealBean = DealsDetailsBean.fromJson(map);

    if (dealBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }

    if (dealBean.status) {

      dealsBean = dealBean;
      dealsListBean = dealBean.data;
      setState(() {
        isLoader = false;
      });
    }
  }




  void callDealDeleteAPI(String index) async{

    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmDealsDelete;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        },
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      dealViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
    Utils.hideKeyboard(context);
  }


}
