import 'dart:convert';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/leads_details_bean.dart';
import 'package:example/Models/notes_details_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Screens/CRM/CreateNoteScreen.dart';
import 'package:example/Screens/CRM/CreateTaskScreen.dart';
import 'package:example/Screens/platform.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'package:transition/transition.dart';

class LeadViewDetailScreen extends StatefulWidget {
  String pageTitle;
  String email;
  String firstName;
  String id;
  Color theme;
  Color secondaryColor;
  String type;
  Function() onCallBack;
  String dateTime;
  LeadsListBean leadListDetails;
  int permissionStr;
  bool visible;

  LeadViewDetailScreen({
    Key key,
    this.pageTitle,
    this.theme,
    this.type,
    this.id,
    this.onCallBack(),
    this.leadListDetails,
    this.secondaryColor,
    this.dateTime,
    this.permissionStr,
    this.visible,
  }) : super(key: key);

  @override
  _LeadViewDetailScreenState createState() => _LeadViewDetailScreenState();
}

class _LeadViewDetailScreenState extends State<LeadViewDetailScreen> {
  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List.empty(growable: true);
  NotesDetailsBean notesDetailsBean;
  List<NotesListDetailsBean> notesListDetailsBean =
      new List.empty(growable: true);
  bool flag = true;
  dynamic formGeneral;
  List<dynamic> status = new List.empty(growable: true);
  List<dynamic> industry = new List.empty(growable: true);
  List<dynamic> leadSource = new List.empty(growable: true);
  List<dynamic> sales = new List.empty(growable: true);
  String form;
  dynamic response;
  bool isLoader = true;
  bool isLoading = true;
  Map keyBoardTypes = {
    'number': TextInputType.number,
    'text': TextInputType.text,
    'email': TextInputType.emailAddress,
    'url': TextInputType.url,
  };

  List<String> taskName = ["Open", "Close"];

  bool showFloating = false;
  int indexValue;

  BuildContext bContext;
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    super.initState();

    leadMastViewAPI();
    initSharedPref();

    if (widget.type != "Add") {
      taskAPI();
      notesAPI();
    }
  }

  String ownerId;

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  @override
  Widget build(BuildContext context) {
    bContext = context;
    return SafeArea(
      child: Scaffold(
        key: _scaffoldKey,
        appBar: widget.type == "Add"
            ? AppBar(
                title: Text("Add Lead"),
                backgroundColor: widget.theme,
              )
            : null,
        body: widget.type != "Add"
            ? DefaultTabController(
                length: 3,
                child: NestedScrollView(
                    headerSliverBuilder:
                        (BuildContext context, bool innerBoxIsScrolled) {
                      return <Widget>[
                        SliverAppBar(
                          expandedHeight: 150.0,
                          floating: true,
                          pinned: true,
                          title: widget.type != "Add"
                              ? Text(widget.leadListDetails.companyname)
                              : Text(widget.pageTitle),
                          backgroundColor:
                              widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                          bottom: TabBar(
                            onTap: (index) {
                              indexValue = index;
                              if (index == 1) {
                                setState(() {
                                  showFloating = true;
                                });
                              } else if (index == 2) {
                                setState(() {
                                  showFloating = true;
                                });
                              } else {
                                setState(() {
                                  showFloating = false;
                                });
                              }
                            },
                            indicatorColor:
                                widget.theme ?? AppColors.APP_COLOR_PRIMARY,
                            labelColor: Colors.black87,
                            unselectedLabelColor: Colors.grey,
                            tabs: [
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.LEAD_INFO,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.TASK,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                              Tab(
                                child: Container(
                                  child: Text(
                                    AppStrings.NOTES,
                                    style: TextStyle(
                                        color: AppColors.APP_COLOR_WHITE,
                                        fontSize: FontSize.FONT_15,
                                        fontWeight: FontWeight.bold),
                                  ),
                                ),
                              ),
                            ],
                          ),
                        ),
                      ];
                    },
                    body: isLoading
                        ? Center(
                            child: Loader(
                            loaderColor: widget.theme,
                          ))
                        : TabBarView(
                            physics: NeverScrollableScrollPhysics(),
                            children: [
                              _formView,
                              RefreshIndicator(
                                  color: widget.theme,
                                  onRefresh: () {
                                    return taskAPI();
                                  },
                                  child: _taskView),
                              RefreshIndicator(
                                  color: widget.theme,
                                  onRefresh: () {
                                    return notesAPI();
                                  },
                                  child: _notesView),
                            ],
                          )),
              )
            : isLoading
                ? Center(
                    child: Loader(
                    loaderColor: widget.theme,
                  ))
                : Container(child: _formView),
        floatingActionButton: showFloating
            ? Visibility(
                visible: widget.visible,
                child: FloatingActionButton(
                  mini: true,
                  onPressed: () {
                    if (indexValue == 1) {
                      Navigator.push(
                          context,
                          Transition(
                              child: CreateTaskScreen(
                                id: widget.id.toString(),
                                primaryColor: widget.theme,
                                secondaryColor: widget.secondaryColor,
                                onCallBack: onTaskAPICall,
                                fromPageId: 'leadid',
                              ),
                              transitionEffect:
                                  TransitionEffect.RIGHT_TO_LEFT));
                    } else {
                      Navigator.push(
                          context,
                          Transition(
                              child: CreateNoteScreen(
                                id: widget.id.toString(),
                                primaryColor: widget.theme,
                                secondaryColor: widget.secondaryColor,
                                onCallBack: onNotesAPICall,
                                fromPageId: 'leadid',
                              ),
                              transitionEffect:
                                  TransitionEffect.RIGHT_TO_LEFT));
                    }
                  },
                  child: const Icon(Icons.add),
                  backgroundColor: widget.theme ?? Colors.green,
                ),
              )
            : null,
      ),
    );
  }

  Widget get _formView {
    return new ListView(
      children: [
        Form(
          child: new Center(
            child: new Column(children: <Widget>[
              new JsonSchema(
                primaryColor: widget.theme ?? AppColors.primaryColor,
                secondaryColor:
                    widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                form: form,
                keyboardTypes: keyBoardTypes,
                onChanged: (dynamic response) {
                  this.response = response;
                },
                buttonText: widget.type == AppStrings.ADD
                    ? AppStrings.ADD
                    : AppStrings.SAVE,
                actionSave: (data) {
                  Utils.hideKeyboard(context);
                  setState(() {
                    formGeneral = data;
                  });

                  if (widget.permissionStr == AppStrings.fullAccessForAll ||
                      widget.permissionStr == AppStrings.fullAccessForOwn) {
                    var jsonObj;
                    if (widget.type != "Add") {
                      jsonObj = {
                        for (int i = 0; i < data['fields'].length; i++)
                          json.encode(
                                  data['fields'][i]['apiKeyValue'].toString()):
                              json.encode(data['fields'][i]['value'].toString())
                      };
                    } else {
                      jsonObj = {
                        for (int i = 0; i < data['fields'].length; i++)
                          json.encode(
                                  data['fields'][i]['apiKeyValue'].toString()):
                              json.encode(
                                  data['fields'][i]['value'].toString()),
                        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                      };
                    }

                    if (widget.type != "Add") {
                      updateLeadsAPI(jsonObj.toString());
                    } else {
                      addLeadsAPI(jsonObj.toString());
                    }
                  } else {
                    Utils.showInSnackBar(_scaffoldKey, AppStrings.noAccess);
                  }
                },
                actionCancel: () {
                  Navigator.pop(context);
                },
                buttonSave: true,
              ),
            ]),
          ),
        )
      ],
    );
  }

  Widget get _taskView {
    if (taskListBeanData.length != 0) {
      return ListView.builder(
        itemCount: taskName.length,
        itemBuilder: (_, index) {
          return ExpansionTile(
              title: Text(
                taskName[index].toString() == "Open"
                    ? "UpComing Task"
                    : taskName[index].toString() == "Close"
                        ? "Completed Task"
                        : taskName[index].toString(),
                style: TextStyle(
                    color: AppColors.APP_COLOR_BLACK,
                    fontSize: FontSize.FONT_17,
                    fontWeight: FontWeight.bold),
              ),
              children: [
                ...taskListBeanData.map((e) {
                  if (e.statusid.name == taskName[index]) {
                    return BUICard.taskCard(
                      cardStatus: e.activitytypeid.name,
                      cardDesc: e.desc,
                      cardDate: widget.dateTime == "dd-mm-yyyy" &&
                              widget.dateTime != null
                          ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(e.duedate)}" ??
                              ""
                          : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(e.duedate)}" ??
                              "",
                      cardType: CardType.cardTask,
                      cardSize: CardSize.medium,
                      cardColor: Colors.white,
                      cardTextColor: Colors.black87,
                      width: MediaQuery.of(context).size.width,
                      padding: 5,
                      height: 140,
                      isActiveStatus: e.statusid.name == "Open" ? false : true,
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(5.0),
                      ),
                      elevation: 10,
                      secondaryColor: widget.secondaryColor,
                      onStateChange: () {
                        setState(() {
                          if (e.statusid.name == "Open") {
                            e.statusid.name = "Close";
                            Utils.showInSnackBar(
                                _scaffoldKey, "Move into Close state");
                          } else {
                            Utils.showInSnackBar(_scaffoldKey,
                                "You cannot change the task from the closed position to the open position. Please contact admin");
                          }
                        });
                      },
                    );
                  } else {
                    return Container();
                  }
                }).toList(),
              ]);
        },
      );
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_TASK_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Widget get _notesView {
    if (notesListDetailsBean.length != 0) {
      return ListView.builder(
          padding: const EdgeInsets.all(8),
          itemCount: notesListDetailsBean.length,
          itemBuilder: (BuildContext context, int index) {
            return BUICard.notesCard(
              cardType: CardType.cardNotes,
              cardSize: CardSize.medium,
              cardColor: Colors.white,
              cardTextColor: Colors.black87,
              width: MediaQuery.of(context).size.width,
              padding: 2,
              height: 120,
              cardAuthor: widget.leadListDetails.firstname,
              cardDesc: notesListDetailsBean[index].desc,
              cardDate: widget.dateTime == "dd-mm-yyyy" &&
                      widget.dateTime != null
                  ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(notesListDetailsBean[index].updatedAt)}" ??
                      ""
                  : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(notesListDetailsBean[index].updatedAt)}" ??
                      "",
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(5.0),
              ),
              elevation: 10,
              secondaryColor: widget.secondaryColor,
              flag: flag,
              onStateChange: () {
                setState(() {
                  flag = !flag;
                });
              },
            );

            /*return BUICard(
              cardType: CardType.customCard,
              cardText: notesListDetailsBean[index].desc.toString(),
              cardSize: CardSize.small,
              height: MediaQuery.of(context).size.height * 0.2,
              cardColor: Colors.white,
              elevation: 20,
              padding: 15,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(15.0),
              ),
            );*/
          });
    } else {
      return Center(
        child: Container(
          child: Text(
            AppStrings.NO_NOTES_AVAILABLE,
            style: TextStyle(
                color: AppColors.textColor,
                fontSize: FontSize.FONT_20,
                fontWeight: FontWeight.bold),
          ),
        ),
      );
    }
  }

  Future<void> leadMastViewAPI() async {
    setState(() {
      isLoading = true;
    });

    String urlController = ServerUrl.crmMasterDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {
      setState(() {
        isLoader = true;
      });

      MyApp.callLoginAPI(context);
    }
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Lead")
            if (detailsBean.data[i].key == "Status")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];
      industry = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Industry")
            {
              'label': detailsBean.data[i].name,
              'value': detailsBean.data[i].id.toString(),
            },
      ];

      leadSource = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Lead")
            if (detailsBean.data[i].key == "Source")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      sales = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Area")
            {
              'label': detailsBean.data[i].name,
              'value': detailsBean.data[i].id.toString(),
            },
      ];

      if (widget.type != "Add") {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'tags',
              'key': 'text',
              'type': 'input',
              'label': 'Project Tag',
              'required': true,
              'readOnly': true,
              'value': widget.leadListDetails.tags.toString() != "null"
                  ? widget.leadListDetails.tags.toString()
                  : "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'value': widget.leadListDetails.statusid.id.toString(),
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'title',
              'key': 'text',
              'type': 'input',
              'label': 'Title',
              'required': true,
              'readOnly': false,
              'value': widget.leadListDetails.title.toString() != "null"
                  ? widget.leadListDetails.title.toString()
                  : "",
            },
            {
              'apiKeyValue': 'firstname',
              'key': 'text',
              'type': 'input',
              'label': 'First Name',
              'required': true,
              'readOnly': false,
              'value': widget.leadListDetails.firstname.toString() != "null"
                  ? widget.leadListDetails.firstname.toString()
                  : "",
            },
            {
              'apiKeyValue': 'lastname',
              'key': 'text',
              'type': 'input',
              'label': 'Last Name',
              'required': true,
              'readOnly': false,
              'value': widget.leadListDetails.lastname.toString() != "null"
                  ? widget.leadListDetails.lastname.toString()
                  : "",
            },
            {
              'apiKeyValue': 'email',
              'key': 'email',
              'type': 'input',
              'label': 'e-mail id',
              'required': true,
              'readOnly': false,
              'value': widget.leadListDetails.email.toString() != "null"
                  ? widget.leadListDetails.email.toString()
                  : "",
            },
            {
              'apiKeyValue': 'phone',
              'key': 'number',
              'type': 'input',
              'label': 'Contact no',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': widget.leadListDetails.phone.toString() != "null"
                  ? widget.leadListDetails.phone.toString()
                  : "",
            },
            {
              'apiKeyValue': 'leadSourceId',
              'key': 'Lead Source',
              'type': 'select',
              'label': 'Lead Source',
              'required': true,
              'value':
                  widget.leadListDetails.industryLeadSourceId.id.toString(),
              'items': leadSource != null
                  ? leadSource
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'companyname',
              'key': 'text',
              'type': 'input',
              'label': 'Company Name',
              'required': true,
              'readOnly': true,
              'value': widget.leadListDetails.companyname.toString() != "null"
                  ? widget.leadListDetails.companyname.toString()
                  : "",
            },
            {
              'apiKeyValue': 'industry',
              'key': 'Industry',
              'type': 'select',
              'label': 'Industry',
              'value': widget.leadListDetails.industry.id.toString(),
              'required': true,
              'items': industry != null
                  ? industry
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'website',
              'key': 'url',
              'type': 'input',
              'label': 'Website',
              'required': true,
              'value': widget.leadListDetails.website.toString() != "null"
                  ? widget.leadListDetails.website.toString()
                  : "",
            },
            {
              'apiKeyValue': 'annualrevenue',
              'key': 'number',
              'type': 'input',
              'label': 'Annual Revenue',
              'required': true,
              'value': widget.leadListDetails.annualrevenue.toString() != "null"
                  ? widget.leadListDetails.annualrevenue.toString()
                  : "",
            },
            {
              'apiKeyValue': 'description',
              'key': 'text',
              'type': 'textInput',
              'label': 'Description',
              'required': false,
              'readOnly': false,
              'value': widget.leadListDetails.description.toString() != "null"
                  ? widget.leadListDetails.description.toString()
                  : "",
            },
            {
              'apiKeyValue': 'salesarea',
              'key': 'Sales',
              'type': 'select',
              'label': 'Sales Area',
              'value': widget.leadListDetails.salesarea.toString(),
              'required': true,
              'items': sales != null
                  ? sales
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'noofemployees',
              'key': 'number',
              'type': 'input',
              'label': 'No of Employees',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': widget.leadListDetails.noofemployees.toString() != "null"
                  ? widget.leadListDetails.noofemployees.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address1',
              'key': 'text',
              'type': 'input',
              'label': 'Address Line 1',
              'required': true,
              'value': widget.leadListDetails.address1.toString() != "null"
                  ? widget.leadListDetails.address1.toString()
                  : "",
            },
            {
              'apiKeyValue': 'address2',
              'key': 'text',
              'type': 'input',
              'label': 'Address Line 2',
              'required': false,
              'value': widget.leadListDetails.address2.toString() != "null"
                  ? widget.leadListDetails.address2.toString()
                  : "",
            },
            {
              'apiKeyValue': 'pincode',
              'key': 'number',
              'type': 'input',
              'label': 'Pincode ',
              'required': true,
              'value': widget.leadListDetails.pincode.toString() != "null"
                  ? widget.leadListDetails.pincode.toString()
                  : "",
            },
            {
              'apiKeyValue': 'state',
              'key': 'text',
              'type': 'input',
              'label': 'State ',
              'required': true,
              'value': widget.leadListDetails.state.toString() != "null"
                  ? widget.leadListDetails.state.toString()
                  : "",
            },
            {
              'apiKeyValue': 'country',
              'key': 'text',
              'type': 'input',
              'label': 'Country ',
              'required': true,
              'value': widget.leadListDetails.country.toString() != "null"
                  ? widget.leadListDetails.country.toString()
                  : "",
            },
          ]
        });
      } else {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'owner_id',
              'key': 'text',
              'type': 'input',
              'label': 'Owner ID',
              'required': true,
              'readOnly': true,
              'value': ownerId ?? "",
              'visibility': false,
            },
            {
              'apiKeyValue': 'updatedby',
              'key': 'text',
              'type': 'input',
              'label': 'Updated By',
              'required': true,
              'readOnly': true,
              'value': ownerId ?? "",
              'visibility': false,
            },
            {
              'apiKeyValue': 'createdby',
              'key': 'text',
              'type': 'input',
              'label': 'Created By',
              'required': true,
              'readOnly': true,
              'value': ownerId ?? "",
              'visibility': false,
            },
            {
              'apiKeyValue': 'tags',
              'key': 'text',
              'type': 'input',
              'label': 'Project Tag',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'title',
              'key': 'text',
              'type': 'input',
              'label': 'Title',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'firstname',
              'key': 'text',
              'type': 'input',
              'label': 'First Name',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'lastname',
              'key': 'text',
              'type': 'input',
              'label': 'Last Name',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'email',
              'key': 'email',
              'type': 'input',
              'label': 'e-mail id',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'phone',
              'key': 'number',
              'type': 'input',
              'label': 'Contact no',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': "",
            },
            {
              'apiKeyValue': 'leadSourceId',
              'key': 'Lead Source',
              'type': 'select',
              'label': 'Lead Source',
              'required': true,
              'items': leadSource != null
                  ? leadSource
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'companyname',
              'key': 'text',
              'type': 'input',
              'label': 'Company Name',
              'required': true,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'industry',
              'key': 'Industry',
              'type': 'select',
              'label': 'Industry',
              'required': true,
              'items': industry != null
                  ? industry
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'website',
              'key': 'url',
              'type': 'input',
              'label': 'Website',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'annualrevenue',
              'key': 'number',
              'type': 'input',
              'label': 'Annual Revenue',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'description',
              'key': 'text',
              'type': 'textInput',
              'label': 'Description',
              'required': false,
              'readOnly': false,
              'value': "",
            },
            {
              'apiKeyValue': 'salesarea',
              'key': 'Sales',
              'type': 'select',
              'label': 'Sales Area',
              'required': true,
              'items': sales != null
                  ? sales
                  : [
                      {'label': '1', 'value': '2'}
                    ]
            },
            {
              'apiKeyValue': 'noofemployees',
              'key': 'number',
              'type': 'input',
              'label': 'No of Employees',
              'required': true,
              'readOnly': false,
              'validator': 'number',
              'value': "",
            },
            {
              'apiKeyValue': 'address1',
              'key': 'text',
              'type': 'input',
              'label': 'Address Line 1',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'address2',
              'key': 'text',
              'type': 'input',
              'label': 'Address Line 2',
              'required': false,
              'value': "",
            },
            {
              'apiKeyValue': 'pincode',
              'key': 'number',
              'type': 'input',
              'label': 'Pincode ',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'state',
              'key': 'text',
              'type': 'input',
              'label': 'State ',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'country',
              'key': 'text',
              'type': 'input',
              'label': 'Country ',
              'required': true,
              'value': "",
            },
          ]
        });
      }

      setState(() {
        isLoader = false;
        isLoading = false;
      });
    } else {
      setState(() {
        isLoader = false;
        isLoading = false;
      });
    }
  }

  Future<void> addLeadsAPI(String jsonString) async {
    String urlController = ServerUrl.crmCreateLead;

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonString.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> updateLeadsAPI(String jsonString) async {
    String urlController = ServerUrl.crmUpdateLead;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": widget.id.toString()}
      ],
      "\"data\"": [jsonString.toString()],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(_scaffoldKey, baseBean.message);
    }
  }

  Future<void> taskAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmTaskViewDetails;
    var objToSend = {
      "\"condition\"": [
        {"\"leadid\"": widget.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (taskDetailsResponseBean.status) {
      taskDetailBean = taskDetailsResponseBean;
      taskListBeanData = taskDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
      Utils.showInSnackBar(_scaffoldKey, taskDetailsResponseBean.message);
    }
  }

  Future<void> notesAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmNotesView;
    var objToSend = {
      "\"condition\"": [
        {"\"leadId\"": widget.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    NotesDetailsBean notesDetailsBeanResponse = NotesDetailsBean.fromJson(map);
    if (notesDetailsBeanResponse.status) {
      notesDetailsBean = notesDetailsBeanResponse;
      notesListDetailsBean = notesDetailsBeanResponse.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onTaskAPICall() async {
    isLoader = false;

    setState(() {
      taskAPI();
      this.mounted;
    });
  }

  Future<void> onNotesAPICall() async {
    isLoader = false;

    setState(() {
      notesAPI();
      this.mounted;
    });
  }
}
