import 'dart:convert';

import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Screens/CRM/AddCreateTaskScreen.dart';
import 'package:example/Screens/CRM/FilterScreen.dart';
import 'package:example/Screens/CRM/TaskViewDetailScreen.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/platform.dart';
import 'package:example/Screens/NoDataFoundScreen.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/BUIAndroidAlertBox.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import 'DrawerMenuHeaderDesign.dart';

class TaskScreen extends DrawerContent {
  TaskScreen({Key key, this.title, this.permissionStr});

  final String title;

  int permissionStr;

  @override
  _TaskScreenState createState() => _TaskScreenState();
}

class _TaskScreenState extends State<TaskScreen> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List.empty(growable: true);

  bool isLoader = true;

  String jsonFormFilterData;

  List<String> actions = ["Edit", "Delete"];

  List<dynamic> account = new List.empty(growable: true);
  List<dynamic> status = new List.empty(growable: true);
  List<dynamic> contact = new List.empty(growable: true);
  List<dynamic> lead = new List.empty(growable: true);
  List<dynamic> activityType = new List.empty(growable: true);
  List<dynamic> deal = new List.empty(growable: true);
  String dateTime;

  bool isLoading = false;

  ScrollController _scrollController = new ScrollController();
  int startIndex = 0;
  int offset = 10;
  bool visible = true;

  @override
  void initState() {
    taskScreenViewDetailsAPI();
    colorCodePref();

    //widget.permissionStr = int.parse("11110", radix: 2);
    _scrollController.addListener(() {
      if (_scrollController.position.pixels ==
          _scrollController.position.maxScrollExtent) {
        startIndex = startIndex + offset;
        taskScreenViewDetailsAPI();
      }
    });
    permissionMethod();
    super.initState();
  }

  var now = DateTime.now();

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: true,
                isFilterClicked: onFilterClicked,
              ),
              RefreshIndicator(
                color: primaryColor,
                onRefresh: () async {
                  startIndex = 0;
                  taskListBeanData = List<TaskDetailList>.empty(growable: true);
                  return await taskScreenViewDetailsAPI();
                },
                child: isLoader
                    ? Stack(
                        children: [
                          deleteDetailsView,
                          isLoaderView,
                        ],
                      )
                    : Stack(
                        children: [deleteDetailsView, detailsView],
                      ),
              )
            ],
          ),
        ),
        floatingActionButton: Visibility(
          visible: visible,
          child: FloatingActionButton(
            onPressed: () {
              Navigator.push(
                  context,
                  Transition(
                      child: AddCreateTaskScreen(
                        primaryColor: primaryColor,
                        secondaryColor: secondaryColor,
                        onCallBack: onCallBackAPI,
                      ),
                      transitionEffect: TransitionEffect.SCALE));
              /* if (widget.permissionStr == AppStrings.fullAccessForAll) {

                Navigator.push(
                    context,
                    Transition(
                        child: AddCreateTaskScreen(
                          primaryColor: primaryColor,
                          secondaryColor: secondaryColor,
                          onCallBack: onCallBackAPI,
                        ),
                        transitionEffect: TransitionEffect.SCALE));
              } else if (widget.permissionStr ==
                  AppStrings.viewAndAddAccessForOwn) {
                Toast.show("viewAndAddAccessForOwn", context);
              } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
                Toast.show("fullAccessForOwn", context);
              } else if (widget.permissionStr ==
                  AppStrings.viewAndAddAccessForAllUser) {
                Toast.show("viewAndAddAccessForAllUser", context);
              } else {
                Toast.show("You have don't access to delete", context);
              }*/
            },
            child: const Icon(Icons.add),
            backgroundColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
          AppStrings.delete_and_edit,


          style: MyApp.textTheme.bodyText1.copyWith(  color: Colors.black87,
              fontSize: FontSize.FONT_12,
              //fontStyle: FontStyle.italic,
              fontWeight: FontWeight.bold,
              letterSpacing: 1,
              wordSpacing: 1
          )
      ),
      color: Colors.grey[400],
    );
  }

  Widget get addDetailsView {
    return Container(
      height: 60,
      padding: EdgeInsets.all(5),
      alignment: Alignment.center,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Container(
            height: 50,
            width: 50,
            padding: EdgeInsets.all(5),
            child: ClipOval(
              child: Material(
                color: primaryColor, // button color
                child: InkWell(
                  splashColor: Colors.white, // inkwell color
                  child: SizedBox(
                      width: 56,
                      height: 56,
                      child: Icon(Icons.add, color: Colors.white)),
                  onTap: () {
                    taskScreenViewDetailsAPI();
                  },
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget get detailsView {
    if (taskListBeanData.length != 0) {
      return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: taskListBeanData.length +
              1, // Add one more item for progress indicator
          padding: EdgeInsets.symmetric(vertical: 8.0),
          itemBuilder: (BuildContext context, int index) {
            if (index == taskListBeanData.length) {
              return _buildProgressIndicator();
            } else {
              return buildUsageItems(context, index);
            }
          },
          controller: _scrollController,
        ),
      );
    } else {
      return NoDataFoundScreen();
    }
  }

  Widget _buildProgressIndicator() {
    return new Padding(
      padding: const EdgeInsets.all(8.0),
      child: new Center(
        child: new Opacity(
          opacity: isLoading ? 1.0 : 00,
          //child: new CircularProgressIndicator(),
          child: Loader(
            loaderColor: primaryColor,
          ),
        ),
      ),
    );
  }

  Widget buildUsageItems(BuildContext context, int index) {
    return Dismissible(
      key: Key(taskListBeanData[index].id.toString()),
      background: slideRightBackground(),
      secondaryBackground: slideLeftBackground(),
      confirmDismiss: (direction) async {
        if (direction == DismissDirection.endToStart) {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this ${taskListBeanData[index].activitytypeid.name}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  callTasksDeleteAPI(taskListBeanData[index].id.toString());
                  taskListBeanData.removeAt(index);
                });
              },
            );
            return res;
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            final bool res = await BUIAndroidAlertBox.showBUIAndroidAlertBox(
              context: context,
              type: "Warning",
              enterWidget: Text(
                  "Are you sure do you want to delete this ${taskListBeanData[index].activitytypeid.name}"),
              isPositiveEnabled: true,
              isNegativeEnabled: true,
              setPositiveAction: () {
                setState(() {
                  callTasksDeleteAPI(taskListBeanData[index].id.toString());
                  taskListBeanData.removeAt(index);
                });
              },
            );
            return res;
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        } else {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            return Navigator.of(context).push(MaterialPageRoute(
              builder: (context) {
                return TaskViewDetailScreen(
                  dateTime: dateTime,
                  pageTitle: "Info",
                  id: taskListBeanData[index].id,
                  taskListBeanData: taskListBeanData[index],
                  theme: primaryColor,
                  secondaryColor: secondaryColor,
                  onCallBack: onCallBackAPI,
                );
              },
            ));
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            return Navigator.of(context).push(MaterialPageRoute(
              builder: (context) {
                return TaskViewDetailScreen(
                  dateTime: dateTime,
                  pageTitle: "Info",
                  id: taskListBeanData[index].id,
                  taskListBeanData: taskListBeanData[index],
                  theme: primaryColor,
                  secondaryColor: secondaryColor,
                  onCallBack: onCallBackAPI,
                );
              },
            ));
          } else {
            Toast.show("You have don't access to delete", context);
            return false;
          }
        }
      },
      child: InkWell(
        onTap: () {
          if (widget.permissionStr == AppStrings.fullAccessForAll) {
            return Navigator.of(context).push(MaterialPageRoute(
              builder: (context) {
                return TaskViewDetailScreen(
                  dateTime: dateTime,
                  pageTitle: "Info",
                  id: taskListBeanData[index].id,
                  taskListBeanData: taskListBeanData[index],
                  theme: primaryColor,
                  secondaryColor: secondaryColor,
                  onCallBack: onCallBackAPI,
                );
              },
            ));
          } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
            return Navigator.of(context).push(MaterialPageRoute(
              builder: (context) {
                return TaskViewDetailScreen(
                  dateTime: dateTime,
                  pageTitle: "Info",
                  id: taskListBeanData[index].id,
                  taskListBeanData: taskListBeanData[index],
                  theme: primaryColor,
                  secondaryColor: secondaryColor,
                  onCallBack: onCallBackAPI,
                );
              },
            ));
          } else {
            Toast.show("You have don't access to Edit", context);
            return false;
          }
        },
        child: BUICard.dashBoardCard(
          width: MediaQuery.of(context).size.width,
          cardSize: CardSize.large,
          cardTitle: taskListBeanData[index].activitytypeid.name != null
              ? taskListBeanData[index].activitytypeid.name
              : "",
          cardName: taskListBeanData[index].duedate != null
              ? taskListBeanData[index].duedate
              : "",
          statusTag: taskListBeanData[index].statusid.name != null
              ? taskListBeanData[index].statusid.name
              : "",
          dashBoardCardTime: dateTime == "dd-mm-yyyy" && dateTime != null
              ? "${MyApp.buiFormatter.buiDateTimeDDMMYYFormat(taskListBeanData[index].createdAt)}" ??
                  ""
              : "${MyApp.buiFormatter.buiDateTimeYYYYMMDDFormat(taskListBeanData[index].createdAt)}" ??
                  "",
          isTagValue: false,
          height: 130,
          elevation: 10,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5.0),
          ),
        ),
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    String dateTme = prefs.getString(AppStrings.date);

    dateTime = dateTme;
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  Future<void> onAPICallBack() async {
    isLoader = false;

    setState(() {
      taskScreenViewDetailsAPI();
      this.mounted;
    });
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Message Pressed");
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Setting Pressed");
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Notification Pressed");
    });
  }

  Future<void> onPopUpMenuPressed(int count) async {
    switch (count) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
      context,
      Transition(
          child: ThemePickerScreen(
            onBackPressed: onBackPressed,
          ),
          transitionEffect: TransitionEffect.SCALE),
    );
  }

  Future<void> onFilterClicked() async {
    colorCodePref();

    colorCodePref();
    setState(() {
      Navigator.push(
          context,
          Transition(
              child: FilterScreen(
                primaryColor: primaryColor,
                secondaryColor: secondaryColor,
                filterFormData: jsonFormFilterData,
                fromPageId: "activityid",
                isActionApply: (String obj) {
                  callFilterAPI(obj);
                },
              ),
              transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
    });
  }

  Future<void> callTasksDeleteAPI(String index) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmDeleteTask;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      taskScreenViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onCallBackAPI() async {
    isLoader = false;

    setState(() {
      taskScreenViewDetailsAPI();
      this.mounted;
    });
  }

  Future<void> taskScreenViewDetailsAPI() async {
    setState(() {
      if (startIndex == 0)
        isLoader = true;
      else
        isLoading = true;
    });
    String urlController = ServerUrl.crmTaskViewDetails;
    var objToSend = {
      "\"condition\"": [],
      /*   "\"page\"": {
        "\"orderby\"": [
          {
            "\"id\"": "\"${"asc"}\"",
          }
        ],
        "\"no\"": "\"${startIndex.toString()}\"",
        "\"size\"": "\"${10}\"",
      },*/
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (startIndex == 0 && taskDetailsResponseBean.data == null) {
      setState(() {
        isLoader = true;
      });
    }

    if (taskDetailsResponseBean.status) {
      taskDetailBean = taskDetailsResponseBean;
      taskListBeanData = taskDetailsResponseBean.data;

      activityType = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].activitytypeid != null)
            {
              'label': taskListBeanData[i].activitytypeid.name,
              'value': taskListBeanData[i].activitytypeid.id.toString(),
            },
      ];
      status = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].statusid != null)
            {
              'label': taskListBeanData[i].statusid.name,
              'value': taskListBeanData[i].statusid.id.toString(),
            },
      ];
      account = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].accountid != null)
            {
              'label': taskListBeanData[i].accountid.companyname,
              'value': taskListBeanData[i].accountid.id.toString(),
            },
      ];
      contact = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].contactid != null)
            {
              'label': taskListBeanData[i].contactid.firstname,
              'value': taskListBeanData[i].contactid.id.toString(),
            },
      ];
      lead = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].leadid != null)
            {
              'label': taskListBeanData[i].leadid.firstname,
              'value': taskListBeanData[i].leadid.id.toString(),
            },
      ];
      deal = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].dealid != null)
            {
              'label': taskListBeanData[i].dealid.name,
              'value': taskListBeanData[i].dealid.id.toString(),
            },
      ];

      account.insert(0, {
        'label': '',
        'value': '',
      });
      contact.insert(0, {
        'label': '',
        'value': '',
      });
      deal.insert(0, {
        'label': '',
        'value': '',
      });
      lead.insert(0, {
        'label': '',
        'value': '',
      });
      activityType.insert(0, {
        'label': '',
        'value': '',
      });
      status.insert(0, {
        'label': '',
        'value': '',
      });

      final statusList = status.map((item) => jsonEncode(item)).toList();

      final uniqueStatusJsonList = statusList.toSet().toList();

      status = uniqueStatusJsonList.map((item) => jsonDecode(item)).toList();

      final activityTypeList =
          activityType.map((item) => jsonEncode(item)).toList();

      final uniqueActivityTypeList = activityTypeList.toSet().toList();

      activityType =
          uniqueActivityTypeList.map((item) => jsonDecode(item)).toList();

      final accountList = account.map((item) => jsonEncode(item)).toList();

      final uniqueAccountList = accountList.toSet().toList();

      account = uniqueAccountList.map((item) => jsonDecode(item)).toList();

      final contactList = contact.map((item) => jsonEncode(item)).toList();

      final uniqueContactList = contactList.toSet().toList();

      contact = uniqueContactList.map((item) => jsonDecode(item)).toList();

      final dealList = deal.map((item) => jsonEncode(item)).toList();

      final uniqueDealList = dealList.toSet().toList();

      deal = uniqueDealList.map((item) => jsonDecode(item)).toList();

      final leadList = lead.map((item) => jsonEncode(item)).toList();

      final uniqueLeadList = leadList.toSet().toList();

      lead = uniqueLeadList.map((item) => jsonDecode(item)).toList();

      print("Deal List====" + deal.toString());

      jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'accountid',
            'key': 'Status',
            'type': 'select',
            'label': 'Account Id',
            'required': false,
            'value': '',
            'items': account != null
                ? account
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'contactid',
            'key': 'Status',
            'type': 'select',
            'label': 'Contact',
            'required': false,
            'value': '',
            'items': contact != null
                ? contact
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'leadid',
            'key': 'Status',
            'type': 'select',
            'label': 'Lead',
            'required': false,
            'value': '',
            'items': lead != null
                ? lead
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'dealid',
            'key': 'Status',
            'type': 'select',
            'label': 'Deal',
            'required': false,
            'value': '',
            'items': deal != null
                ? deal
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'activitytypeid',
            'key': 'Status',
            'type': 'select',
            'label': 'Activity Type',
            'required': false,
            'value': '',
            'items': activityType != null
                ? activityType
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'Status',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'value': '',
            'items': status != null
                ? status
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
        ],
      });

      setState(() {
        if (startIndex == 0)
          isLoader = false;
        else
          isLoading = false;
      });
    }
  }

  Future<void> callFilterAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.crmTaskViewDetails;
    var objToSend = {
      "\"condition\"": [obj],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
        MyPlatformState.isLoader = true;
      });
      MyApp.callLoginAPI(context);
    }

    if (taskDetailsResponseBean.status) {
      taskDetailBean = taskDetailsResponseBean;
      taskListBeanData = taskDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }
}
