import 'dart:convert';

import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Constants/SvgPictures.dart';
import 'package:example/Constants/fontSize.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/module_bean.dart';
import 'package:example/Screens/CRM/DealScreen.dart';
import 'package:example/Screens/CRM/MenuPage.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/PMO/DrawerContent.dart';
import 'package:example/Screens/ERP/TestScreen.dart';
import 'package:example/Screens/SignUpScreen.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:transition/transition.dart';
import 'package:universal_platform/universal_platform.dart';
import 'package:google_fonts/google_fonts.dart';
import '../main.dart';
import 'Admin/MenuPage.dart';

class ModuleSelectionScreen extends StatefulWidget {
  const ModuleSelectionScreen({Key key}) : super(key: key);

  @override
  ModuleSelectionScreenState createState() => ModuleSelectionScreenState();
}

class ModuleSelectionScreenState extends State<ModuleSelectionScreen> {
  SharedPreferences prefs;

  Color primaryColor = AppColors.APP_COLOR_PRIMARY;

  bool isLoader = true;

  List<Modules> moduleName = new List.empty(growable: true);

  ModuleBean moduleBean;

  static List<Screens> screens = new List.empty(growable: true);

  String userID;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();


    if(UniversalPlatform.isWeb){

      callModuleScreenAPI();

    }
    else{
      colorCodePref();
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    userID = prefs.getString(AppStrings.USER_ID);

    callModuleScreenAPI();
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }
  }

  bool isEnabled = true;

  double radius = 10;
  double width = 160;
  double height = 160;
  double elevation = 5;

  @override
  Widget build(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;
    return SafeArea(
      child: Scaffold(
          appBar: AppBar(
            automaticallyImplyLeading: false,
            elevation: 1.0,
            backgroundColor: Colors.white,
            title: Text(AppStrings.selectModule,
                style: textTheme.bodyText1.copyWith(color: AppColors.APP_COLOR_BLACK,letterSpacing: 1,wordSpacing: 1,fontWeight: FontWeight.bold)),
            actions: <Widget>[
              Container(
                margin: EdgeInsets.only(right: 8.0),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.center,
                  crossAxisAlignment: CrossAxisAlignment.center,
                  children: <Widget>[
                    Container(
                      padding: EdgeInsets.all(8),
                      height: 120,
                      child: Image.asset(ImageAssets.LOGO_WITH_NAME),
                    ),
                    IconButton(
                      icon: Icon(Icons.color_lens),
                      color: primaryColor,
                      onPressed: () {
                        Navigator.push(
                          context,
                          Transition(
                              child: ThemePickerScreen(
                                onBackPressed: colorCodePref,
                              ),
                              transitionEffect: TransitionEffect.SCALE),
                        );
                      },
                    ),
                    IconButton(
                      icon: Icon(Icons.logout),
                      color: primaryColor,
                      onPressed: () {
                        MainWidgetState.signOutFunction(context);
                      },
                    ),
                  ],
                ),
              )
            ],
          ),
          //backgroundColor: Colors.black,
          body: isLoader
              ? _isLoaderView
              : RefreshIndicator(
                  backgroundColor: primaryColor,
                  color: Colors.white,
                  onRefresh: () {
                    return callModuleScreenAPI();
                  },
                  child: GridView.builder(
                      gridDelegate: SliverGridDelegateWithMaxCrossAxisExtent(
                          maxCrossAxisExtent: 200,
                          childAspectRatio: 3 / 2,
                          crossAxisSpacing: 1,
                          mainAxisSpacing: 1),
                      itemCount: moduleName.length,
                      itemBuilder: (BuildContext ctx, index) {
                        return InkWell(
                          radius: 30,
                          enableFeedback: true,
                          splashColor: AppColors.halfWhite,
                          onTap: () {
                            for (int i = 0;
                                i < moduleBean.data.screens.length;
                                i++) {
                              if (moduleName[index].id ==
                                  moduleBean.data.screens[i].moduleId) {
                                screens = moduleBean.data.screens;
                              }
                            }

                            if (moduleName[index].moduleName.toString() ==
                                "Crm") {
                              Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (context) {
                                    return MainWidget(id: moduleName[index].id);
                                  },
                                ),
                              );
                            } else if (moduleName[index]
                                    .moduleName
                                    .toString() ==
                                "Pmo") {
                              Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (context) {
                                    return MainWidgetPMO(
                                        id: moduleName[index].id);
                                  },
                                ),
                              );
                            } else if (moduleName[index]
                                    .moduleName
                                    .toString() ==
                                "Admin") {
                             /* Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (context) {
                                    return DealScreen(
                                      permissionStr: 31,
                                    );
                                  },
                                ),
                              );*/

                              Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (context) {
                                    return MainWidgetAdmin(
                                        id: moduleName[index].id);
                                  },
                                ),
                              );
                            } else {
                              Navigator.of(context).push(
                                MaterialPageRoute(
                                  builder: (context) {
                                    return MainWidgetPMO(
                                        id: moduleName[index].id);
                                  },
                                ),
                              );
                            }
                          },
                          child: Container(
                              alignment: Alignment.center,
                              padding: EdgeInsets.all(8),
                              width: width,
                              height: height,
                              child: BUICard.imageWithTextCard(
                                isVisible: true,
                                cardText: moduleName[index].moduleName,
                                //cardText: moduleName[index].moduleName.length > 0 ?'${moduleName[index].moduleName[0].toUpperCase()}${moduleName[index].moduleName.substring(1)}':'',
                                cardTextColor: AppColors.primaryColor,
                                cardType: CardType.cardWithImageAndText,
                                sourceImage: SvgPictures.FB_SVG,
                                cardColor: primaryColor,
                                borderRadius: 1,
                                radius: 5,
                               // textStyle: textTheme.bodyText1.copyWith(fontWeight: FontWeight.bold,fontSize: 15,letterSpacing: 1,color: AppColors.appWhite),
                                shape: RoundedRectangleBorder(
                                  borderRadius: BorderRadius.circular(5.0),
                                ),
                              )),
                        );
                      }),
                )),
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Future<void> callModuleScreenAPI() async {
    setState(() {
      isLoader = true;
    });

    String urlController = ServerUrl.buiUserPermission;

    if(UniversalPlatform.isWeb){
      userID = SignUpScreenState.user_uid;
    }

    var objToSend = {
      "\"condition\"": [
        {
          "\"${"uid"}\"": "\"${userID.toString()}\"",
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ModuleBean moduleBeanResponse = ModuleBean.fromJson(map);

    if (moduleBeanResponse.status) {
      moduleBean = moduleBeanResponse;
      moduleName = moduleBeanResponse.data.modules;

      setState(() {
        isLoader = false;
      });
    }
  }
}
