import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Screens/CRM/ThemePickerScreen.dart';
import 'package:example/Screens/CommonViewDetailScreen.dart';
import 'package:example/Screens/PMO/DrawerContent.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:transition/transition.dart';

import '../ModuleSelectionScreen.dart';

class AdminPage extends DrawerContent {
  String title;
  int permissionStr;

  AdminPage({Key key, this.title, this.permissionStr});

  @override
  _AdminPageState createState() => _AdminPageState();
}

class _AdminPageState extends State<AdminPage> with TickerProviderStateMixin {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;
  SharedPreferences prefs;
  String ownerId;

  final List<Tab> myTabs = <Tab>[
    Tab(text: AppStrings.users),
    Tab(text: AppStrings.roles),
    Tab(text: AppStrings.permission),
    Tab(text: AppStrings.tech),
    Tab(text: AppStrings.timesheetExp),
  ];

  String jsonForm;
  dynamic formGeneral;
  dynamic response;

  bool isLoader = true;

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  Map keyBoardTypes = {
    'input': TextInputType.number,
    'text': TextInputType.text,
  };

  String label = AppStrings.users;

  TabController controller;

  bool visible = true;

  @override
  void initState() {
    super.initState();

    controller = new TabController(
      length: myTabs.length,
      vsync: this,
    );

    masterView();

    //_mainTab = TabController(vsync: this, length: myTabs.length);
    colorCodePref();
    permissionMethod();
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });

    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColorPref;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: DefaultTabController(
        length: myTabs.length,
        child: Scaffold(
          appBar: AppBar(
            backgroundColor: primaryColor,
            leading: InkWell(
                onTap: () {
                  widget.onMenuPressed();
                },
                child: Icon(Icons.menu)),
            bottom: PreferredSize(
                child: TabBar(
                  controller: controller,
                  isScrollable: true,
                  labelColor: AppColors.APP_COLOR_WHITE,
                  indicatorColor: secondaryColor,
                  unselectedLabelColor: Colors.white.withOpacity(0.6),
                  tabs: myTabs,
                  //  controller: _mainTab,
                ),
                preferredSize: Size.fromHeight(30.0)),
            actions: <Widget>[
              Container(
                  //padding: EdgeInsets.only(right: 2),
                  child: PopupMenuButton(
                elevation: 30,
                //offset: Offset(0, 100),
                tooltip: "More Feature",
                itemBuilder: (context) => MainWidgetPMOState.popUpMenuItem,
                onCanceled: () {},
                onSelected: (value) {
                  onPopUpMenuFunction(value);
                },
                icon: Icon(
                  Icons.more_vert_sharp,
                  color: AppColors.appWhite,
                ),
              )),
            ],
          ),
          key: _scaffoldKey,
          body: isLoader
              ? _isLoaderView
              : ListView(
                  children: [
                    Container(
                      height: MediaQuery.of(context).size.height,
                      child: TabBarView(
                        //physics: NeverScrollableScrollPhysics(),
                        controller: controller,
                        children: myTabs.map((Tab tab) {
                          setState(() {
                            label = tab.text;
                          });

                          return Center(
                              child: Container(
                            child: Text(label),
                          ));
                          /*  return Column(
                              children: [deleteDetailsView, _leaveList]);*/
                        }).toList(),
                      ),
                    ),
                  ],
                ),
          floatingActionButton: Visibility(
            visible: visible,
            child: FloatingActionButton(
              mini: true,
              onPressed: () {
                print(controller.index.toString());
                switch (controller.index) {
                  case 0:
                    Navigator.push(
                        context,
                        Transition(
                            child: CommonViewDetailScreen(
                              primaryColor: primaryColor,
                              secondaryColor: secondaryColor,
                              pageId: AppStrings.users,
                              ownerId: ownerId ?? "7",
                              commonDD: [1, 2, 3],
                              title: AppStrings.create + AppStrings.users,
                              permissionStr: widget.permissionStr,
                              onCallBack: () {},
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;
                  case 1:
                    Navigator.push(
                        context,
                        Transition(
                            child: CommonViewDetailScreen(
                              primaryColor: primaryColor,
                              secondaryColor: secondaryColor,
                              pageId: AppStrings.roles,
                              ownerId: ownerId ?? "7",
                              title: AppStrings.create + AppStrings.roles,
                              commonDD: [1, 2, 3],
                              onCallBack: () {},
                              permissionStr: widget.permissionStr,
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;
                  case 2:
                    Navigator.push(
                        context,
                        Transition(
                            child: CommonViewDetailScreen(
                              primaryColor: primaryColor,
                              secondaryColor: secondaryColor,
                              pageId: AppStrings.permission,
                              ownerId: ownerId ?? "7",
                              title: AppStrings.create + AppStrings.permission,
                              commonDD: [1, 2, 3],
                              onCallBack: () {},
                              permissionStr: widget.permissionStr,
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;
                  case 3:
                    Navigator.push(
                        context,
                        Transition(
                            child: CommonViewDetailScreen(
                              primaryColor: primaryColor,
                              secondaryColor: secondaryColor,
                              pageId: AppStrings.tech,
                              ownerId: ownerId ?? "7",
                              title: AppStrings.create + AppStrings.tech,
                              commonDD: [1, 2, 3],
                              onCallBack: () {},
                              permissionStr: widget.permissionStr,
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;
                  case 4:
                    Navigator.push(
                        context,
                        Transition(
                            child: CommonViewDetailScreen(
                              primaryColor: primaryColor,
                              secondaryColor: secondaryColor,
                              pageId: AppStrings.timesheetExp,
                              ownerId: ownerId ?? "7",
                              title:
                                  AppStrings.create + AppStrings.timeSheetPage,
                              commonDD: [1, 2, 3],
                              onCallBack: () {},
                              permissionStr: widget.permissionStr,
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;

                  default:
                    Navigator.push(
                        context,
                        Transition(
                            child: ThemePickerScreen(
                              onBackPressed: onBackPressed,
                            ),
                            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
                    break;
                }
              },
              child: const Icon(
                Icons.add,
              ),
              backgroundColor: primaryColor ?? Colors.green,
            ),
          ),
        ),
      ),
    );
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        AppStrings.delete_and_edit,
        style: TextStyle(
            fontSize: 14, color: Colors.black87, fontWeight: FontWeight.bold),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Widget slideLeftBackground() {
    return Container(
      color: Colors.red,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.end,
          children: <Widget>[
            Icon(
              Icons.delete,
              color: Colors.white,
            ),
            Text(
              " Delete",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.right,
            ),
            SizedBox(
              width: 20,
            ),
          ],
        ),
        alignment: Alignment.centerRight,
      ),
    );
  }

  Widget slideRightBackground() {
    return Container(
      color: Colors.green,
      child: Align(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.start,
          children: <Widget>[
            SizedBox(
              width: 20,
            ),
            Icon(
              Icons.edit,
              color: Colors.white,
            ),
            Text(
              " Edit",
              style: TextStyle(
                color: Colors.white,
                fontWeight: FontWeight.w700,
              ),
              textAlign: TextAlign.left,
            ),
          ],
        ),
        alignment: Alignment.centerLeft,
      ),
    );
  }

  Future<void> masterView() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoMaster;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);
    if (detailsBean.status) {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> applyLeaveAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoLeaveCreate;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: obj.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      setState(() {
        isLoader = false;
      });
    } else {
      Toast.show(baseBean.message.toString(), context);
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onPopUpMenuFunction(int s) async {
    switch (s) {
      case 1:
        onSwitchModuleScreen();
        break;
      case 2:
        onSettingFunction();
        break;
      case 3:
        onNotificationFunction();
        break;
      case 4:
        onMessageFunction();
        break;
      case 5:
        onThemeFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onSwitchModuleScreen() async {
    Navigator.push(
        context,
        Transition(
            child: ModuleSelectionScreen(),
            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        Transition(
            child: ThemePickerScreen(
              onBackPressed: onBackPressed,
            ),
            transitionEffect: TransitionEffect.RIGHT_TO_LEFT));
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {});
  }

  Future<void> onNotificationFunction() async {
    setState(() {});
  }

  void permissionMethod() {
    if (widget.permissionStr == AppStrings.fullAccessForAll) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForOwn) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.fullAccessForOwn) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.viewAccessForAllUser) {
      setState(() {
        visible = false;
      });
    } else if (widget.permissionStr == AppStrings.viewAndAddAccessForAllUser) {
      setState(() {
        visible = true;
      });
    } else if (widget.permissionStr == AppStrings.noAccessPermission) {
      setState(() {
        visible = false;
      });
    } else {
      setState(() {
        visible = false;
      });
    }
  }
}
