import 'dart:convert';

import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/fontFamily.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/PMO/time_sheet_bean.dart';
import 'package:example/Screens/CRM/DrawerMenuHeaderDesign.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class DashBoardPage extends DrawerContent {
  String title;

  DashBoardPage({Key key, this.title});

  @override
  _DashBoardPageState createState() => _DashBoardPageState();
}

class _DashBoardPageState extends State<DashBoardPage> {
  Color primaryColor = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;
  SharedPreferences prefs;
  bool isLoader = true;

  TimeSheetBean timeSheetReponse;

  List<TimeSheetListBean> timeSheetListBean = new List.empty(growable: true);

  dynamic sumValue = 0;

  List<dynamic> totalHoursArray = new List.empty(growable: true);

  @override
  void initState() {
    super.initState();
    colorCodePref();
    timeSheetList();
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: isLoader ? _isLoaderView : _detailsView,
      ),
    );
  }

  Widget get _detailsView {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.center,
      mainAxisAlignment: MainAxisAlignment.start,
      children: [
        DrawerMenuDesign(
          onMenuClicked: widget.onMenuPressed,
          isFilterEnabled: false,
        ),
        RefreshIndicator(
          color: primaryColor,
          onRefresh: () async {
            return await timeSheetList();
          },
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisAlignment: MainAxisAlignment.center,
            mainAxisSize: MainAxisSize.max,
            children: [
              SizedBox(
                height: MediaQuery.of(context).size.height * 0.25,
              ),
              Container(
                alignment: Alignment.center,
                child: Text(
                  AppStrings.welcomeToAagniaWorkBench,
                  style: TextStyle(
                    fontWeight: FontWeight.bold,
                    fontFamily: FontFamily.Gilroy,
                    fontSize: FontSize.FONT_22,
                    fontStyle: FontStyle.italic,
                    letterSpacing: 1.0,
                    wordSpacing: 1.0,
                    color: AppColors.APP_COLOR_BLACK,
                  ),
                  textAlign: TextAlign.center,
                ),
              ),
              SizedBox(
                height: 50,
              ),
              Text(
                AppStrings.thisMonth +
                    MyApp.buiFormatter.getCurrentMonthName().toString(),
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontFamily: FontFamily.Gilroy,
                  fontSize: FontSize.FONT_18,
                  fontStyle: FontStyle.italic,
                  letterSpacing: 1.0,
                  wordSpacing: 1.0,
                  color: AppColors.APP_COLOR_BLACK,
                ),
                textAlign: TextAlign.center,
              ),
              SizedBox(
                height: 10,
              ),
              Text(
                AppStrings.totalHours + sumValue.toString() + AppStrings.hours,
                style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontFamily: FontFamily.Gilroy,
                  fontSize: FontSize.FONT_18,
                  fontStyle: FontStyle.italic,
                  letterSpacing: 1.0,
                  wordSpacing: 1.0,
                  color: AppColors.APP_COLOR_BLACK,
                ),
                textAlign: TextAlign.center,
              ),
            ],
          ),
        )
      ],
    );
  }

  Widget get _isLoaderView {
    return Center(
      child: Loader(
        loaderColor: primaryColor,
      ),
    );
  }

  Future<void> timeSheetList() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.pmoTimeSheetViewDetails;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TimeSheetBean timeSheetBean = TimeSheetBean.fromJson(map);

    if (timeSheetBean.status) {
      timeSheetReponse = timeSheetBean;
      timeSheetListBean = timeSheetBean.data;

      for (int i = 0; i < timeSheetBean.data.length; i++) {
        totalHoursArray.add(timeSheetBean.data[i].hours);
        sumValue += (timeSheetBean.data[i].hours);
      }
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        primaryColor = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return secondaryColorPref;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }
}
