/*
    Refer License for Terms & Conditions
 */
library bui_component;

import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/AppStrings.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class BUIAndroidAlertBox {
  static Future showBUIAndroidAlertBox({
    BuildContext context,
    Widget enterWidget,
    String type,
    bool isPositiveEnabled,
    bool isNegativeEnabled = false,
    Color positiveButtonColor,
    Color positiveButtonTextColor,
    Function() setPositiveAction,
  }) {
    assert(context != null, "context is null!!");
    assert(type != null, "Type is null!!");
    assert(enterWidget != null, "willDisplayWidget is null!!");
    final textTheme = Theme.of(context).textTheme;
    return showDialog(
        context: context,
        builder: (context) {
          return AlertDialog(
            title: Text(
              type ?? 'Warning!!',
              textAlign: TextAlign.center,
              style: textTheme.bodyText1.copyWith(
                color: Colors.black87,
                fontSize: 20,
                fontWeight: FontWeight.bold,
                wordSpacing: 1,
                letterSpacing: 1,
              ),

            ),
            shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.all(Radius.circular(15)),
            ),
            backgroundColor: Colors.white,
            content: Column(mainAxisSize: MainAxisSize.min, children: <Widget>[
              enterWidget,
              SizedBox(
                height: 20,
              ),

              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  isNegativeEnabled
                      ? Button.mini(
                    width: MediaQuery.of(context).size.width * 0.15,
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                    buttonSize: ButtonSize.medium,
                    buttonType: ButtonType.custom,
                    btnColor: AppColors.APP_BUTTON_GREY,
                    btnTextColor: positiveButtonColor,
                    btnText: AppStrings.cancel,
                    elevation: 10,
                  )
                      : Container(),
                  isNegativeEnabled
                      ? SizedBox(
                    width: 30,
                  )
                      : Container(),
                  isPositiveEnabled
                      ? Button.mini(
                    width: MediaQuery.of(context).size.width * 0.1,
                    onPressed: () {
                      setPositiveAction();
                      Navigator.of(context).pop();
                    },
                    buttonSize: ButtonSize.medium,
                    buttonType: ButtonType.custom,
                    btnColor:
                    positiveButtonColor ?? AppColors.errorWarning,
                    btnTextColor: AppColors.appWhite,
                    btnText: AppStrings.okay,
                    elevation: 10,
                  )
                      : Button.mini(
                    width: MediaQuery.of(context).size.width * 0.15,
                    onPressed: () {
                      setPositiveAction();
                      Navigator.of(context).pop();
                    },
                    buttonSize: ButtonSize.medium,
                    buttonType: ButtonType.custom,
                    btnColor:
                    positiveButtonColor ?? AppColors.errorWarning,
                    btnTextColor: AppColors.appWhite,
                    btnText: AppStrings.okay,
                    elevation: 10,
                  )
                ],
              ),
            ]),
            elevation: 1,
          );
        });
  }
}


